
import os
import numpy as np
from matplotlib import pyplot
import spm1d

# load HDF5 data (30 rows, 100 columns):
dir0         = os.path.dirname(__file__)
fname        = os.path.join(dir0, 'data', 'ex_kinematics.h5')
Y            = spm1d.io.load(fname)   #30 curves, 100 nodes

# plot:
pyplot.close('all')
pyplot.plot(Y.T, color = 'k')
pyplot.xlabel('Time (%)', size=20)
pyplot.ylabel(r'$\theta$ (deg)', size=20)
pyplot.show()




import os
import numpy as np


def get_datafilepath():
	return os.path.join( os.path.dirname(__file__), 'datafiles' )


class _Dataset(object):
	def __init__(self):
		self.STAT     = 'Z'
		self.design   = None   #design string (e.g. "One-way ANOVA")
		self.dim      = 0      #data dimensionality (0 or 1)
		self.Y        = None   #dataset
		self.z        = None   #expected test stat
		self.df       = None   #expected degrees of freedom
		self.p        = None   #expected p value
		self.cite     = None   #literature citation (if relevant)
		self.datafile = None   #data file (if available on the web)
		self.www      = None   #web source
		self.note     = None   #note
		self._set_values()
	def __repr__(self):
		s      = 'Dataset\n'
		s     += '   Name      : "%s"\n' %self.__class__.__name__
		s     += '   Design    :  %s\n' %self.design
		s     += '   Data dim  :  %d\n' %self.dim
		if self.cite:
			s += '   Reference :  %s\n' %self.cite
		if self.www:
			s += '   Web       :  %s\n' %self.www
		if self.datafile:
			s += '   Data file :  %s\n' %self.datafile
		if self.note:
			s += '   %s :  %s\n' %tuple(self.note)
		ss     = self.get_expected_results_as_string()
		s     += ss 
		return s
	def _set_values(self):    #abstract method;  instantiated by all subclasses
		pass
	def get_dependent_variable(self):
		return self.Y
	def get_expected_df(self):
		return self.df
	def get_data(self):
		return self.Y
	def get_expected_test_stat(self):
		return self.z
	def get_expected_p_value(self):
		return self.p
	def get_expected_results_as_string(self):
		s      = '  (Expected results)\n'
		s     += '  %s          :  %s\n' %(self.STAT, str(self.z))
		s     += '  df         :  %s\n' %str(self.df)
		s     += '  p          :  %s\n' %str(self.p)
		return s




# class _Dataset0D(_Dataset):
# 	pass
class Dataset1D(_Dataset):
	def __init__(self):
		super(Dataset1D, self).__init__()
		self.dim       = 1


class _DatasetANOVA(_Dataset):
	def __init__(self):
		super(_DatasetANOVA, self).__init__()
		self.STAT   = 'F'
class _DatasetT(_Dataset):
	def __init__(self):
		super(_DatasetT, self).__init__()
		self.STAT   = 't'
class _DatasetT2(_Dataset):
	def __init__(self):
		super(_DatasetT2, self).__init__()
		self.STAT   = 'T2'
class _DatasetX2(_Dataset):
	def __init__(self):
		super(_DatasetX2, self).__init__()
		self.STAT   = 'X2'



class DatasetANOVA1(_DatasetANOVA):
	def __init__(self):
		self.rm       = False  #repeated measures
		super(DatasetANOVA1, self).__init__()
		self.design = 'One-way ANOVA'
	def get_data(self):
		return self.Y, self.A
	# def get_expected_df(self, type='sphericity_assumed'):
	# 	if type=='sphericity_assumed':
	# 		return self.df
	# 	if type=='GG':
	# 		return self.dfGG
	# 	elif type=='GGX':
	# 		return self.dfGGX
	# 	elif type=='HF':
	# 		return self.dfHF
	# def get_expected_p_value(self, type='sphericity_assumed'):
	# 	if type=='sphericity_assumed':
	# 		return self.p
	# 	if type=='GG':
	# 		return self.pGG
	# 	elif type=='GGX':
	# 		return self.pGGX
	# 	elif type=='HF':
	# 		return self.pHF
	
class DatasetANOVA1rm(_DatasetANOVA):
	def __init__(self):
		self.rm       = True  #repeated measures
		super(DatasetANOVA1rm, self).__init__()
		self.design = 'One-way repeated measures ANOVA'
	def get_data(self):
		return self.Y, self.A, self.SUBJ

class DatasetANOVA2(_DatasetANOVA):
	def __init__(self):
		self.rm       = False  #repeated measures
		super(DatasetANOVA2, self).__init__()
		self.design = 'Two-way ANOVA'
	def get_data(self):
		return self.Y, self.A, self.B
class DatasetANOVA2nested(DatasetANOVA2):
	def __init__(self):
		super(DatasetANOVA2nested, self).__init__()
		self.design = 'Two-way ANOVA (nested)'
class DatasetANOVA2rm(DatasetANOVA2):
	def __init__(self):
		self.rm       = True  #repeated measures
		super(DatasetANOVA2rm, self).__init__()
		self.design = 'Two-way repeated measures ANOVA'
	def get_data(self):
		return self.Y, self.A, self.B, self.SUBJ
class DatasetANOVA2onerm(DatasetANOVA2rm):
	def __init__(self):
		super(DatasetANOVA2onerm, self).__init__()
		self.design = 'Two-way ANOVA (repeated measures on one factor)'



class DatasetANOVA3(_DatasetANOVA):
	def __init__(self):
		self.rm       = False  #repeated measures
		super(DatasetANOVA3, self).__init__()
		self.design = 'Three-way ANOVA'
	def get_data(self):
		return self.Y, self.A, self.B, self.C
class DatasetANOVA3nested(DatasetANOVA3):
	def __init__(self):
		super(DatasetANOVA3nested, self).__init__()
		self.design = 'Three-way ANOVA (nested)'
class DatasetANOVA3onerm(DatasetANOVA3):
	def __init__(self):
		self.rm       = True  #repeated measures
		super(DatasetANOVA3onerm, self).__init__()
		self.design = 'Three-way ANOVA (repeated measures on one factor)'
	def get_data(self):
		return self.Y, self.A, self.B, self.C, self.SUBJ
class DatasetANOVA3tworm(DatasetANOVA3onerm):
	def __init__(self):
		super(DatasetANOVA3tworm, self).__init__()
		self.design = 'Three-way ANOVA (repeated measures on two factors)'





class DatasetCCA(_DatasetX2):
	def __init__(self):
		super(DatasetCCA, self).__init__()
		self.design = "Canonical Correlation Analysis"
	def get_data(self):
		return self.Y, self.x



class DatasetHotellings1(_DatasetT2):
	def __init__(self):
		super(DatasetHotellings1, self).__init__()
		self.design = "One-sample Hotelling's T2 test"
	def get_data(self):
		return self.Y, self.mu
class DatasetHotellings2(_DatasetT2):
	def __init__(self):
		super(DatasetHotellings2, self).__init__()
		self.design = "Two-sample Hotelling's T2 test"
	def get_data(self):
		return self.YA, self.YB
class DatasetHotellingsPaired(DatasetHotellings2):
	def __init__(self):
		super(DatasetHotellingsPaired, self).__init__()
		self.design = "Paired Hotelling's T2 test"



class DatasetMANOVA1(_DatasetX2):
	def __init__(self):
		super(DatasetMANOVA1, self).__init__()
		self.design = "One-way MANOVA"
	def get_data(self):
		return self.Y, self.A







class DatasetT1(_DatasetT):
	def __init__(self):
		self.mu     = None
		super(DatasetT1, self).__init__()
		self.design = 'One-sample t test'
	def get_data(self):
		return self.Y, self.mu

class DatasetT2(_DatasetT):
	def __init__(self):
		self.YA     = None
		self.YB     = None
		self.A      = None
		super(DatasetT2, self).__init__()
		self.design = 'Two-sample t test'
	def get_data(self):
		return self.YA, self.YB

class DatasetTpaired(DatasetT2):
	def __init__(self):
		super(DatasetTpaired, self).__init__()
		self.design = 'Paired t test'


class DatasetRegress(_DatasetT):
	def __init__(self):
		super(DatasetRegress, self).__init__()
		self.design = 'Simple linear regression'
	def get_data(self):
		return self.Y, self.x
	def get_expected_results_as_string(self):
		s      = '  (Expected results)\n'
		s     += '   %s         :  %s\n' %(self.STAT, self.z)
		s     += '   df        :  (%s, %s)\n' %self.df
		s     += '   r         :  %s\n' %(self.r)
		s     += '   p         :  %s\n' %(self.p)
		return s










import numpy as np
import _mvbase
import _spm




def _T2_onesample_singlenode(y):  #at a single node:
	y        = np.matrix(y)
	n        = y.shape[0]      #nResponses
	m        = y.mean(axis=0)  #mean vector
	W        = np.cov(y.T)     #covariance
	T2       = n * m * np.linalg.inv(W) * m.T
	return float(T2)

def _T2_twosample_singlenode(yA, yB):  #at a single node:
	JA,JB    = yA.shape[0], yB.shape[0]  #nResponses
	yA,yB    = np.matrix(yA), np.matrix(yB)
	mA,mB    = yA.mean(axis=0), yB.mean(axis=0)  #means
	WA,WB    = np.cov(yA.T), np.cov(yB.T)
	W        = ((JA-1)*WA + (JB-1)*WB) / (JA+JB-2)
	T2       = (JA*JB)/float(JA+JB)  * (mB-mA) * np.linalg.inv(W) * (mB-mA).T
	return float(T2)




def hotellings(Y, mu=None):
	'''
	One-sample Hotelling's T2 test.
	
	:Parameters:
		- *Y* --- (J x Q x I) numpy array
		- *mu* --- scalar or (Q x I) array (datum)

	
	:Returns:
		- T2 : An **spm1d._spm.SPM_T2** instance
	'''
	
	if Y.ndim==2:
		if mu is not None:
			Y         = Y - mu
		T2            = _T2_onesample_singlenode(Y)
		J,I           = Y.shape
		m,p           = float(J)-1, float(I)
		v1,v2         = p, m
		return _spm.SPM0D_T2(T2, (v1, v2))
	else:
		if mu is not None:
			Y         = Y - mu
		nResponses,nNodes,nVectDim  = Y.shape
		T2            = np.array([_T2_onesample_singlenode(Y[:,i,:])   for i in range(nNodes)])
		R             = _mvbase._get_residuals_onesample(Y)
		W             = _mvbase._fwhm(R)
		rCounts       = _mvbase._resel_counts(R, W)
		m,p           = float(nResponses)-1, float(nVectDim)
		v1,v2         = p, m
		return _spm.SPM_T2(T2, (v1, v2), W, rCounts, None, None, R)
	


def hotellings_paired(YA, YB):
	'''
	Paired Hotelling's T2 test.
	
	:Parameters:
		- *YA* --- (J x Q x I) numpy array
		- *YB* --- (J x Q x I) numpy array

	
	:Returns:
		- T2 : An **spm1d._spm.SPM_T2** instance
		
	:Note:
		- A paired Hotelling's test on (YA,YB) is equivalent to a one-sample Hotelling's test on (YB-YA)
	'''
	
	return hotellings( YB - YA )



def hotellings2(YA, YB, equal_var=True):
	'''
	Two-sample Hotelling's T2 test.
	
	:Parameters:
		- *YA* --- (J x Q x I) numpy array
		- *YB* --- (J x Q x I) numpy array

	
	:Returns:
		- T2 : An **spm1d._spm.SPM_T2** instance
		
	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	if YA.ndim==2:
		T2            = _T2_twosample_singlenode(YA, YB)
		JA,IA         = YA.shape
		JB,IB         = YB.shape
		# v1,v2         = float(IA), float(JA+JB-IA-1)  ###incorrect;  these are F df, not T2 df
		v1,v2         = float(IA), float(JA+JB-2)
		return _spm.SPM0D_T2(T2, (v1, v2))
	else:
		JA,QA,IA      = YA.shape
		JB,QB,IB      = YB.shape
		T2            = np.array([_T2_twosample_singlenode(YA[:,i,:], YB[:,i,:])   for i in range(QA)])
		R             = _mvbase._get_residuals_twosample(YA, YB)
		W             = _mvbase._fwhm(R)
		rCounts       = _mvbase._resel_counts(R, W)
		# v1,v2         = float(IA), float(JA+JB-IA-1)  ###incorrect;  these are F df, not T2 df
		v1,v2         = float(IA), float(JA+JB-2)
		return _spm.SPM_T2(np.array(T2), (v1, v2), W, rCounts, None, None, R)






import contrasts
import designs
import factors
import modelbuilder
import solvers
import util

from ui import anova1,anova1rm
from ui import anova2,anova2nested,anova2rm,anova2onerm
from ui import anova3,anova3nested,anova3onerm,anova3tworm





import os
from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset      = spm1d.data.mv1d.hotellings_paired.Pataky2014cop()
YA,YB        = dataset.get_data()  #A:slow, B:fast
print dataset



#(1) Conduct test:
alpha        = 0.05
T2           = spm1d.stats.hotellings_paired(YA, YB)
T2i          = T2.inference(0.05)



#(2) Plot:
pyplot.close('all')
ax0     = pyplot.subplot(221)
ax1     = pyplot.subplot(222)
ax3     = pyplot.subplot(224)
## plot SPM results:
ax0.plot(YA[:,:,0].T, 'k', label='slow')
ax0.plot(YB[:,:,0].T, 'r', label='fast')
ax1.plot(YA[:,:,1].T, 'k')
ax1.plot(YB[:,:,1].T, 'r')
T2i.plot(ax=ax3)
pyplot.show()







import numpy as np
from matplotlib import pyplot


from factors import Factor,FactorNested,FactorNested2,FactorRM,FactorSubject
from modelbuilder import ModelBuilder


class _Design(object):
	def _get_column_const(self):
		return np.matrix( np.ones(self.J) ).T


	def plot(self, ax=None, plot_contrasts=True, contrastnums=[0,1,2]):
		if plot_contrasts:
			ax0 = pyplot.axes([0.05,0.05,0.4,0.9])
		else:
			ax0 = pyplot.axes()
		# self.design.plot(ax=ax0)
		ax0.imshow(self.X, interpolation='nearest', cmap='gray', vmin=-1, vmax=1, aspect='auto')
		xskip   = self.X.shape[1] / 10 + 1
		yskip   = self.X.shape[0] / 10 + 1
		pyplot.setp(ax0, xticks=range(0, self.X.shape[1], xskip), yticks=range(0, self.J, yskip))
		### plot contrasts:
		if plot_contrasts and self.contrasts!=None:
			yy  = np.linspace(0.7, 0.05, 3)
			if len(self.contrasts) < 4:
				for i,contrast in enumerate(self.contrasts):
					ax  = pyplot.axes([0.53,yy[i],0.45,0.2])
					contrast.plot(ax=ax)
			else:
				for i,ci in enumerate(contrastnums):
					ax  = pyplot.axes([0.53,yy[i],0.45,0.2])
					self.contrasts[ci].plot(ax=ax)
		pyplot.show()
		







class ANOVA1(_Design):
	def __init__(self, A):
		self.X          = None       #design matrix
		self.A          = Factor(A)  #factor levels
		self.J          = self.A.J   #number of observations
		self.contrasts  = None       #contrast matrices
		self._assemble()
		
	

	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XCONST         = self._get_column_const()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'CONST'])
		model.add_main_columns('A', XA)
		model.add_main_columns('CONST', XCONST)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast('A')]



	def set_simplified_design(self, with_const=True, difference_contrasts=True):
		# if self.unbalanced:
		# 	raise( ValueError('ANOVA error: unable to simplify unbalanced designs') )
		# self._set_simplified_design_matrix(with_const)
		# self._set_simplified_contrast()
		if with_const:
			XA             = self.A.get_design_main(simplified=True)
			XCONST         = self._get_column_const()
			model          = ModelBuilder(labels=['A', 'CONST'])
			model.add_main_columns('A', XA)
			model.add_main_columns('CONST', XCONST)
		else:
			XA             = self.A.get_design_main(simplified=True)
			model          = ModelBuilder(labels=['A'])
			model.add_main_columns('A', XA)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		if difference_contrasts:
			self.contrasts = [model.get_contrast_difference('A')]
		else:
			self.contrasts = [model.get_contrast('A')]







class ANOVA1rm(_Design):
	def __init__(self, A, SUBJ):
		self.X          = None          #design matrix
		self.S          = FactorSubject(SUBJ)  #subjects
		self.A          = FactorRM(A, self.S)  #factor levels
		self.J          = self.A.J      #number of observations
		self.contrasts  = None          #contrast matrices
		self._check_balanced()
		self._assemble()


	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XCONST         = self._get_column_const()
		XSA            = self.A.get_design_subject_pooled()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'CONST'], labels_subj=['SA'])
		model.add_main_columns('A', XA)
		model.add_main_columns('CONST', XCONST)
		model.add_subj_columns('SA', XSA)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast('A')]

	def _check_balanced(self):
		if not (self.A.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.A):
			raise( ValueError('Design must be balanced.') )







class ANOVA2(_Design):
	def __init__(self, A, B):
		self.X          = None       #design matrix
		self.A          = Factor(A)  #factor level vector
		self.B          = Factor(B)  #factor level vector
		self.J          = self.A.J   #number of observations
		self.contrasts  = None
		self.balanced   = True
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XAB            = self.A.get_design_interaction(self.B)
		XCONST         = self._get_column_const()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'B', 'AB', 'CONST'])
		model.add_main_columns('A',  XA)
		model.add_main_columns('B',  XB)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('CONST', XCONST)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast(s)  for s in ['A','B','AB']]

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced):
			self.balanced = False
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			self.balanced = False
			raise( ValueError('Design must be balanced.') )





class ANOVA2nested(ANOVA2):
	def __init__(self, A, B):
		self.X          = None
		self.A          = Factor(A)
		self.B          = FactorNested(B, self.A)
		self.J          = self.A.J
		self.contrasts  = None
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XCONST         = self._get_column_const()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'B', 'CONST'])
		model.add_main_columns('A',  XA)
		model.add_main_columns('B',  XB)
		model.add_main_columns('CONST', XCONST)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast(s)  for s in ['A','B']]

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced_nested(self.B):
			raise( ValueError('Design must be balanced.') )



class ANOVA2rm(ANOVA2):
	'''Both A and B are RM factors.'''
	def __init__(self, A, B, SUBJ):
		self.X          = None
		self.S          = FactorSubject(SUBJ)
		self.A          = FactorRM(A, self.S)
		self.B          = FactorRM(B, self.S)
		self.J          = self.A.J
		self.contrasts  = None
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XCONST          = self._get_column_const()
		XSA             = self.A.get_design_subject_partitioned()
		XSB             = self.B.get_design_subject_partitioned()
		XSAB            = self.A.get_design_subject_interaction(self.B)
		XSpooled        = self.A.get_design_subject_pooled()
		### specify model and add design matrix columns:
		model           = ModelBuilder(labels=['A', 'B', 'AB', 'CONST'], labels_subj=['SA', 'SB', 'SAB', 'SPOOLED'])
		model.add_main_columns('A', XA)
		model.add_main_columns('B', XB)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('CONST', XCONST)
		model.add_subj_columns('SA', XSA)
		model.add_subj_columns('SB', XSB)
		model.add_subj_columns('SAB', XSAB)
		model.add_subj_columns('SPOOLED', XSpooled)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast_compound(s, ss)  for s,ss in [('A','SA'),('B','SB'),('AB','SAB')]]
		
	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.A):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )

class ANOVA2onerm(ANOVA2rm):
	'''Only B is an RM factor.'''
	def _assemble(self):
		### assemble design matrix columns:
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XCONST          = self._get_column_const()
		XSApooled       = self.A.get_design_subject_pooled()
		XSB             = self.B.get_design_subject_partitioned()
		### specify model and add design matrix columns:
		model           = ModelBuilder(labels=['A', 'B', 'AB', 'CONST'], labels_subj=['SApooled', 'SB'])
		model.add_main_columns('A', XA)
		model.add_main_columns('B', XB)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('CONST', XCONST)
		model.add_subj_columns('SApooled', XSApooled)
		model.add_subj_columns('SB', XSB)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast_compound(s, ss)  for s,ss in [('A','SApooled'),('B','SB'),('AB','SB')]]


	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.A):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.B):
			raise( ValueError('Design must be balanced.') )













class ANOVA3(_Design):
	def __init__(self, A, B, C):
		self.X          = None       #design matrix
		self.A          = Factor(A)  #factor level vector
		self.B          = Factor(B)  #factor level vector
		self.C          = Factor(C)  #factor level vector
		self.J          = self.A.J   #number of observations
		self.contrasts  = None
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XC             = self.C.get_design_main()
		XAB            = self.A.get_design_interaction(self.B)
		XAC            = self.A.get_design_interaction(self.C)
		XBC            = self.B.get_design_interaction(self.C)
		XABC           = self.A.get_design_interaction_3way(self.B, self.C)
		XCONST         = self._get_column_const()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'B', 'C', 'AB', 'AC', 'BC', 'ABC', 'CONST'])
		model.add_main_columns('A',  XA)
		model.add_main_columns('B',  XB)
		model.add_main_columns('C',  XC)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('AC', XAC)
		model.add_main_columns('BC', XBC)
		model.add_main_columns('ABC', XABC)
		model.add_main_columns('CONST', XCONST)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast(s)  for s in ['A', 'B', 'C', 'AB', 'AC', 'BC', 'ABC']]

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )





class ANOVA3nested(ANOVA3):
	def __init__(self, A, B, C):
		self.X          = None
		self.A          = Factor(A)
		self.B          = FactorNested(B, self.A)
		self.C          = FactorNested2(C, self.B)
		self.J          = self.A.J
		self.contrasts  = None
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XC             = self.C.get_design_main()
		XCONST         = self._get_column_const()
		### specify model and add design matrix columns:
		model          = ModelBuilder(labels=['A', 'B', 'C', 'CONST'])
		model.add_main_columns('A',  XA)
		model.add_main_columns('B',  XB)
		model.add_main_columns('C',  XC)
		model.add_main_columns('CONST', XCONST)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		self.contrasts = [model.get_contrast(s)  for s in ['A','B', 'C']]

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced_nested3(self.B, self.C):
			raise( ValueError('Design must be balanced.') )



class ANOVA3rm(ANOVA3):
	'''A, B and C are all RM factors (not yet implemented).'''
	pass



class ANOVA3onerm(ANOVA3rm):
	'''Only C is an RM factor.'''
	def __init__(self, A, B, C, SUBJ):
		self.X          = None
		self.S          = FactorSubject(SUBJ)
		self.A          = FactorRM(A, self.S)
		self.B          = FactorRM(B, self.S)
		self.C          = FactorRM(C, self.S)
		self.J          = self.A.J
		self.contrasts  = None
		self._swap      = self.A.n > self.B.n   ### swap A and B factors if needed
		self._check_balanced()
		self._swapAB()
		self._assemble()
		self._swapAB()
			

	def _assemble(self):
		### assemble design matrix columns:
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XC              = self.C.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XAC             = self.A.get_design_interaction(self.C)
		XBC             = self.B.get_design_interaction(self.C)
		XABC            = self.A.get_design_interaction_3way(self.B, self.C)
		XCONST          = self._get_column_const()
		XSpooled        = self.A.get_design_subject_pooled()
		XSC             = self.C.get_design_subject_partitioned()
		### specify model and add design matrix columns:
		model           = ModelBuilder(labels=['A','B','C', 'AB','AC','BC', 'ABC', 'CONST'], labels_subj=['Spooled', 'SC'])
		model.add_main_columns('A', XA)
		model.add_main_columns('B', XB)
		model.add_main_columns('C', XC)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('AC', XAC)
		model.add_main_columns('BC', XBC)
		model.add_main_columns('ABC', XABC)
		model.add_main_columns('CONST', XCONST)
		model.add_subj_columns('Spooled', XSpooled)
		model.add_subj_columns('SC', XSC)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		if self._swap:
			sspairs        = [('B','Spooled'), ('A','Spooled'), ('C','SC')]
			sspairs       += [('AB','Spooled'), ('BC','SC'), ('AC','SC')]
			sspairs       += [('ABC','SC')]
		else:
			sspairs        = [('A','Spooled'), ('B','Spooled'), ('C','SC')]
			sspairs       += [('AB','Spooled'), ('AC','SC'), ('BC','SC')]
			sspairs       += [('ABC','SC')]
		self.contrasts = [model.get_contrast_compound(s, ss)  for s,ss in sspairs]

	def _swapAB(self):
		if self._swap:
			A,B         = self.B, self.A
			self.A      = A
			self.B      = B

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.C):
			raise( ValueError('Design must be balanced.') )




class ANOVA3tworm(ANOVA3rm):
	'''Both B and C are RM factors.'''
	def __init__(self, A, B, C, SUBJ):
		self.X          = None
		self.S          = FactorSubject(SUBJ)
		self.A          = FactorRM(A, self.S)
		self.B          = FactorRM(B, self.S)
		self.C          = FactorRM(C, self.S)
		self.J          = self.A.J
		self.contrasts  = None
		self._swap      = self.B.n > self.C.n   ### swap B and C factors if needed
		self._check_balanced()
		self._swapBC()
		self._assemble()
		self._swapBC()
		
		
	def _assemble(self):
		### assemble design matrix columns:
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XC              = self.C.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XAC             = self.A.get_design_interaction(self.C)
		XBC             = self.B.get_design_interaction(self.C)
		XABC            = self.A.get_design_interaction_3way(self.B, self.C)
		XCONST          = self._get_column_const()
		XSpooled        = self.A.get_design_subject_pooled()
		XSB             = self.B.get_design_subject_partitioned()
		XSC             = self.C.get_design_subject_partitioned()
		XSBC            = self.B.get_design_subject_partitioned3(self.C)
		### specify model and add design matrix columns:
		model           = ModelBuilder(labels=['A','B','C', 'AB','AC','BC', 'ABC', 'CONST'], labels_subj=['Spooled', 'SB', 'SC', 'SBC'])
		model.add_main_columns('A', XA)
		model.add_main_columns('B', XB)
		model.add_main_columns('C', XC)
		model.add_main_columns('AB', XAB)
		model.add_main_columns('AC', XAC)
		model.add_main_columns('BC', XBC)
		model.add_main_columns('ABC', XABC)
		model.add_main_columns('CONST', XCONST)
		model.add_subj_columns('Spooled', XSpooled)
		model.add_subj_columns('SB', XSB)
		model.add_subj_columns('SC', XSC)
		model.add_subj_columns('SBC', XSBC)
		### assemble design matrix and contrasts:
		self.X         = model.get_design_matrix()
		if self._swap:
			sspairs    = [('A','Spooled'), ('C','SC'), ('B','SB')]
			sspairs   += [('AC','SC'), ('AB','SB'), ('BC','SBC')]
			sspairs   += [('ABC', 'SBC')]
		else:
			sspairs    = [('A','Spooled'), ('B','SB'), ('C','SC')]
			sspairs   += [('AB','SB'), ('AC','SC'), ('BC','SBC')]
			sspairs   += [('ABC', 'SBC')]

		self.contrasts = [model.get_contrast_compound(s, ss)  for s,ss in sspairs]


	def _swapBC(self):
		if self._swap:
			B,C         = self.C, self.B
			self.B      = B
			self.C      = C

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.C):
			raise( ValueError('Design must be balanced.') )








import numpy as np
from matplotlib import pyplot




class Contrast(object):
	def __init__(self, C):
		self.C          = np.matrix(C)     #contrast matrix
		self.isrm       = False            #repeated measures flag
		self.isub       = False            #unbalanced flag

	def plot(self, ax=None):
		ax  = pyplot.gca() if ax==None else ax
		ax.imshow(self.C, interpolation='nearest', cmap='gray', vmin=-1, vmax=1, aspect='auto')
		xskip   = self.C.shape[1] / 10 + 1
		yskip   = self.C.shape[0] / 10 + 1
		pyplot.setp(ax, xticks=range(0, self.C.shape[1], xskip), yticks=range(0, self.C.shape[0], yskip))


class ContrastUnbalanced(Contrast):
	def __init__(self, C):
		self.C          = C                #list of columns for unbalanced regression
		self.isub       = True             #repeated measures flag
	def plot(self):
		pass


class CompoundContrast(Contrast):
	def __init__(self, C, n):
		self.C          = np.matrix(C)     #contrast matrix
		self.isrm       = True             #repeated measures flag
		self.n          = int(n)
	
	def get_compound_matrices(self):
		return self.C, self.C[self.n:]
	





import numpy as np
from .. import _base




class RSFlavor(_base.DatasetT2):
	def _set_values(self):
		self.www  = 'http://www.real-statistics.com/students-t-distribution/two-sample-t-test-equal-variances/'
		self.YA   = np.array([34,52,17,45,5,29,31,6,52,64])
		self.YB   = np.array([12,8,6,16,12,14,10,18,4,11])
		self.Y    = np.hstack([self.YA, self.YB])
		self.A    = np.array([0]*15 + [1]*15)
		self.z    = 3.454842
		self.df   = 1,9.83928
		self.p    = 0.003162



class ColumbiaPlacebo(_base.DatasetT2):
	def _set_values(self):
		self.www  = 'http://www.stat.columbia.edu/~martin/W2024/R2.pdf'
		self.YA   = np.array([101, 110, 103, 93, 99, 104])
		self.YB   = np.array([91, 87, 99, 77, 88, 91])
		self.Y    = np.hstack([self.YA, self.YB])
		self.A    = np.array([0]*15 + [1]*15)
		self.z    = 3.4456
		self.df   = 1,9.48
		self.p    = 0.003391
















import numpy as np
from scipy import stats
import spm1d




#(0) Load dataset:
dataset = spm1d.data.uv0d.regress.RSRegression()
dataset = spm1d.data.uv0d.regress.ColumbiaHeadCircumference()
y,x     = dataset.get_data()
print dataset


#(1) Conduct t test using spm1d:
spmt    = spm1d.stats.regress(y, x)
spmti   = spmt.inference(0.05, two_tailed=True)
print spmti


#(2) Compare to scipy.stats result:
slope,intercept,r,p,se = stats.linregress(x, y)
t       = r * ((y.size-2)/(1-r*r) )**0.5
print 'scipy.stats result:\n   r = %.5f\n   t = %.5f\n   p = %.5f' %(r,t,p)





# coding: utf-8

import os
import numpy as np
from .. import _base




class PlantarArchAngle(_base.DatasetT2, _base.Dataset1D):
	def _set_values(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'ex_kinematics.npy')
		self.cite     = 'Caravaggi, P., Pataky, T., Günther, M., Savage, R., & Crompton, R. (2010). Dynamics of longitudinal arch support in relation to walking speed: contribution of the plantar aponeurosis. Journal of Anatomy, 217(3), 254–261. http://doi.org/10.1111/j.1469-7580.2010.01261.x'
		Y             = np.load(self.datafile)
		self.YA       = Y[10:20]
		self.YB       = Y[20:]
		self.z        = None
		self.df       = None
		self.p        = None

class SimulatedTwoLocalMax(_base.DatasetT2, _base.Dataset1D):
	def _set_values(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'ex_sim_twolocalmax.npy')
		Y             = np.load(self.datafile)
		self.YA       = Y[:6]
		self.YB       = Y[6:]
		self.z        = None
		self.df       = None
		self.p        = None













import numpy as np
from matplotlib import pyplot
import spm1d

# create random data:
np.random.seed(0)
Y0           = np.random.randn(5,101)

# smooth:
Y           = spm1d.util.smooth(Y0, fwhm=5.0)

# plot:
pyplot.close('all')
pyplot.figure(figsize=(8,3.5))
ax0          = pyplot.axes((0.1,0.15,0.35,0.8))
ax1          = pyplot.axes((0.55,0.15,0.35,0.8))
ax0.plot(Y0.T, 'k')
ax1.plot(Y.T, 'k')
ax0.text(0.5, 0.9, 'Before smoothing', size=14, transform=ax0.transAxes, ha='center')
ax1.text(0.5, 0.9, 'After smoothing', size=14, transform=ax1.transAxes, ha='center')
pyplot.show()




'''
Data checking tools.

(This and all modules whose names start with underscores
are not meant to be accessed directly by the user.)
'''

# Copyright (C) 2014  Todd Pataky
# _datachecks.py version: 0.2.0004 (2014/06/11)

import warnings
import numpy as np



def asmatrix(Y):
	Y = np.asarray(Y)
	return np.matrix(Y).T if Y.ndim==1 else np.matrix(Y)




class SPM1DError(ValueError):
	pass

class DataChecker(object):
	def check(self):
		pass
	
	def check_1d(self, x, argnum):
		ndim      = np.ndim(x)
		if ndim!=1:
			msg   = '\n\nArgument number %d must be a 1D array or list. \n\n' %argnum
			raise SPM1DError(msg)

	def check_2d(self, Y):
		pass
		# ndim      = np.ndim(Y)
		# if ndim!=2:
		# 	msg   = '\n\n%d-D array detected. spm1d only accepts 2D arrays. \n\n' %np.ndim(Y)
		# 	raise SPM1DError(msg)
	
	def check_array(self, Y):
		if not isinstance(Y, np.ndarray):
			msg   = 'Object of type %s detected. Must submit a numpy array.' %type(Y)
			raise SPM1DError(msg)
	
	def check_equal_J(self, Y0, Y1):
		J0,J1     = Y0.shape[0], Y1.shape[0]
		if J0!=J1:
			msg   = 'Unequal number of responses in (J x Q) arrays (J1=%d, J2=%d). J1 must equal J2.' %(J0,J1)
			raise SPM1DError(msg)

	def check_equal_Q(self, Y0, Y1):
		Q0,Q1     = Y0.shape[1], Y1.shape[1]
		if Q0!=Q1:
			msg   = 'Unequal number of nodes in (J x Q) arrays (Q1=%d, Q2=%d). Q1 must equal Q2.' %(Q0,Q1)
			raise SPM1DError(msg)
	
	def check_equal_JQ(self, Y0, Y1):
		self.check_equal_J(Y0, Y1)
		self.check_equal_Q(Y0, Y1)

	def check_size(self, Y):
		pass
		# if Y.shape[1] < 11:
		# 	msg   = '(J x Q) array with Q=%d detected. Q must be at least 11.' %Y.shape[1]
		# 	raise SPM1DError(msg)
			
	
	def check_zero_variance(self, Y, only_warning=False):
		a         = Y.var(axis=0)==0
		if np.any(a):
			ind   = np.argwhere( a ).flatten().tolist()
			msg   = '\n\nZero variance detected at the following nodes:\n\n %s \n' %ind.__repr__()
			if only_warning:
				warnings.warn(msg)
			else:
				raise SPM1DError(msg)




class DataCheckerANOVA1(DataChecker):
	def __init__(self, YY):
		self.YY   = YY
	def check(self):
		[self.check_array(Y)  for Y in self.YY]
		[self.check_2d(Y) for Y in self.YY]
		[self.check_size(Y) for Y in self.YY]
		[self.check_zero_variance(Y) for Y in self.YY]
		nGroups  = len(self.YY)
		for i in range(1, nGroups):
			self.check_equal_Q(self.YY[0], self.YY[i])


class DataCheckerANOVA2(DataChecker):
	def __init__(self, Y, A, B):
		self.Y   = Y
		self.A   = A
		self.B   = B
	def check(self):
		YAB      = self.Y, self.A, self.B
		[self.check_array(x)  for x in YAB]
		self.check_2d(self.Y)
		self.check_1d(self.A, 2)
		self.check_1d(self.B, 3)
		self.check_size(self.Y)
		self.check_equal_J(self.Y, self.A)
		self.check_equal_J(self.Y, self.B)
		self.check_zero_variance(self.Y)



class DataCheckerRegress(DataChecker):
	def __init__(self, Y, x):
		self.Y    = Y
		self.x    = x
	def check(self):
		self.check_array(self.Y)
		self.check_2d(self.Y)
		self.check_1d(self.x, 2)
		self.check_size(self.Y)
		self.check_zero_variance(self.Y)
		self.check_equal_J(self.Y, self.x)
		

class DataCheckerTtest(DataChecker):
	def __init__(self, Y, y0):
		self.Y    = Y
		self.y0   = y0
	def check(self):
		self.check_array(self.Y)
		self.check_2d(self.Y)
		self.check_size(self.Y)
		self.check_zero_variance(self.Y)

class DataCheckerTtest2(DataChecker):
	def __init__(self, YA, YB):
		self.YA   = YA
		self.YB   = YB
	def check(self):
		YY        = self.YA, self.YB
		[self.check_array(Y)  for Y in YY]
		[self.check_2d(Y) for Y in YY]
		[self.check_size(Y) for Y in YY]
		self.check_equal_Q(self.YA, self.YB)
		[self.check_zero_variance(Y) for Y in YY]

class DataCheckerTtestPaired(DataCheckerTtest2):
	def check(self):
		DataCheckerTtest2.check(self)
		self.check_equal_J(self.YA, self.YB)





def check(testname, *args):
	if testname == 'anova1':
		YY       = args[0]
		checker  = DataCheckerANOVA1(YY)
	if testname == 'anova2':
		Y,A,B    = args
		checker  = DataCheckerANOVA2(Y, A, B)
	if testname == 'ttest':
		Y,y0     = args
		checker  = DataCheckerTtest(Y, y0)
	elif testname == 'ttest_paired':
		YA,YB    = args
		checker  = DataCheckerTtestPaired(YA, YB)
	elif testname == 'ttest2':
		YA,YB    = args
		checker  = DataCheckerTtest2(YA, YB)
	elif testname == 'regress':
		Y,x      = args
		checker  = DataCheckerRegress(Y, x)
	checker.check()







import numpy as np
from .. import _base




class RSRegression(_base.DatasetRegress):
	def _set_values(self):
		self.www  = 'http://www.real-statistics.com/regression/hypothesis-testing-significance-regression-line-slope/'
		self.x    = np.array([5, 23, 25, 48, 17, 8, 4, 26, 11, 19, 14, 35, 29, 4, 23])
		self.Y    = np.array([80, 78, 60, 53, 85, 84, 73, 79, 81, 75, 68, 72, 58, 92, 65])
		self.z    = -3.67092
		self.df   = 1,13
		self.p    = 0.002822
		self.r    = -0.71343


class ColumbiaHeadCircumference(_base.DatasetRegress):
	def _set_values(self):
		self.www  = 'http://www.stat.columbia.edu/~martin/W2024/R4.pdf'
		self.x    = np.array([27.75, 24.5, 25.5, 26, 25, 27.75, 26.5, 27, 26.75, 26.75, 27.5])
		self.Y    = np.array([17.5, 17.1, 17.1,17.3, 16.9, 17.6, 17.3, 17.5, 17.3, 17.5, 17.5])
		self.mu   = 0.3
		self.z    = 6.63
		self.df   = 1,9
		self.p    = 9.59e-5
		self.r    = 0.8301**0.5


















from matplotlib import pyplot
import spm1d



# load dataset:
dataset = spm1d.data.uv1d.anova1.SpeedGRFcategorical()
Y,A     = dataset.get_data()
Y0      = Y[A==1]
Y1      = Y[A==2]
Y2      = Y[A==3]


# plot:
pyplot.close('all')
spm1d.plot.plot_mean_sd(Y0, linecolor='b', facecolor=(0.7,0.7,1), edgecolor='b', label='Slow')
spm1d.plot.plot_mean_sd(Y1, label='Normal')
spm1d.plot.plot_mean_sd(Y2, linecolor='r', facecolor=(1,0.7,0.7), edgecolor='r', label='Fast')
pyplot.xlim(0, 100)
pyplot.xlabel('Time (%)', size=20)
pyplot.ylabel(r'$\theta$ (deg)', size=20)
pyplot.legend(loc='lower left')
pyplot.show()



import spm1d




#(0) Load dataset:
dataset = spm1d.data.mv0d.cca.FitnessClub()
dataset = spm1d.data.mv0d.cca.StackExchange()
y,x     = dataset.Y, dataset.x
print dataset



#(1) Conduct T2 test using spm1d:
X2      = spm1d.stats.cca(y, x)
X2i     = X2.inference(alpha=0.05)
print X2i








import numpy as np
from matplotlib import pyplot, cm as colormaps
import _datachecks, _reml, _spm
import rft1d


rank   = np.linalg.matrix_rank
eps    = np.finfo(float).eps   #smallest float, used to avoid divide-by-zero errors




def glm(Y, X, c, Q=None):
	'''
	General linear model (for t contrasts).
	
	:Parameters:
	
	- *Y* --- (J x Q) numpy array (dependent variable)
	- *X* --- (J x B) design matrix  (J responses, B parameters)
	- *c* --- B-component contrast vector (list or array)
	- *Q* --- non-sphericity specifiers (not currently supported for **glm**)
	
	.. note:: Non-sphericity estimates are not supported for **spm1d.stats.glm**
	
	:Returns:
	
	- An **spm1d._spm.SPM_T** object.
	
	:Example:
	
	>>> t  = spm1d.stats.glm(Y, X, (-1,1))
	>>> ti = t.inference(alpha=0.05, two_tailed=True)
	>>> ti.plot()
	'''
	### assemble data:
	Y      = np.matrix(Y)
	X      = np.matrix(X)
	c      = np.matrix(c).T
	### solve the GLM:
	b      = np.linalg.pinv(X)*Y    #parameters
	eij    = Y - X*b                #residuals
	R      = eij.T*eij              #residuals: sum of squares
	df     = Y.shape[0] - rank(X)   #degrees of freedom
	sigma2 = np.diag(R)/df          #variance
	### compute t statistic
	t      = np.array(c.T*b).flatten()  /   (np.sqrt(sigma2*float(c.T*(np.linalg.inv(X.T*X))*c)) + eps)
	### estimate df due to non-sphericity:
	if Q!=None:
		df = _reml.estimate_df_T(Y, X, eij, Q)
	eij    = np.asarray(eij)
	if Y.shape[1] > 1:
		### estimate field smoothness and geometry:
		fwhm   = rft1d.geom.estimate_fwhm(eij)
		resels = rft1d.geom.resel_counts(eij, fwhm, element_based=False)
		t      = _spm.SPM_T(t, (1,df), fwhm, resels, np.asarray(X), np.asarray(b), eij)
	else:
		t      = _spm.SPM0D_T(t, (1,df))
	return t





def regress(Y, x):
	'''
	Simple linear regression.
	
	:Parameters:
	
	- *Y* --- (J x Q) numpy array (dependent variable)
	- *x* --- J-component list or array (independent variable)
	
	:Returns:
	
	- An **spm1d._spm.SPM_T** object.
	
	:Example:
	
	>>> Y  = np.random.rand(10, 101)
	>>> Y  = spm1d.util.smooth(Y, fwhm=10)
	>>> x  = np.random.rand(10)
	>>> t  = spm1d.stats.regress(Y, x)
	>>> ti = t.inference(alpha=0.05)
	>>> ti.plot()
	
	:Notes:
		- the correlation coefficient is retrievable as "t.r" where "t" is the output from **spm1d.stats.regress**
		- statistical inferences are based on *t*, not on *r*
	'''
	Y              = _datachecks.asmatrix(Y)
	_datachecks.check('regress', Y, x)
	J              = Y.shape[0]
	X              = np.ones((J,2))
	X[:,0]         = x
	c              = [1,0]
	spmt           = glm(Y, X, c)
	spmt.r         = spmt.z / (  (J - 2 + spmt.z**2)**0.5)  #t = r * ((J-2)/(1-r*r) )**0.5
	spmt.isregress = True
	return spmt






def ttest(Y, y0=None):
	'''
	One-sample t test.
	
	:Parameters:
	
	- *Y* --- (J x Q) data array  (J responses, Q nodes)
	- *y0* --- optional Q-component datum array (default is the null continuum)
	
	:Returns:
	
	- An **spm1d._spm.SPM_T** object.
	
	:Example:
	
	>>> Y  = np.random.randn(8, 101)
	>>> Y  = spm1d.util.smooth(Y, fwhm=15)
	>>> t  = spm1d.stats.ttest(Y)
	>>> ti = t.inference(alpha=0.05, two_tailed=True)
	>>> ti.plot()
	'''
	Y       = _datachecks.asmatrix(Y)
	_datachecks.check('ttest', Y, y0)
	J       = Y.shape[0]
	Ytemp   = Y.copy()
	if y0!=None:
		Ytemp -= y0
	X       = np.ones((J,1))
	c       = (1)
	### compute SPM{t}:
	return glm(Ytemp, X, c)



def ttest_paired(YA, YB):
	'''
	Paired t test.
	
	:Parameters:
	
	- *YA* --- (J x Q) data array  (J responses, Q nodes)
	- *YB* --- (J x Q) data array  (J responses, Q nodes)

	:Returns:
	
	- An **spm1d._spm.SPM_T** object.
	
	:Example:
	
	>>> YA,YB  = np.random.randn(8, 101), np.random.randn(8, 101)
	>>> YA,YB  = spm1d.util.smooth(Y, fwhm=10), spm1d.util.smooth(Y, fwhm=10)
	
	>>> t      = spm1d.stats.ttest_paired(YA, YB)
	>>> ti = t.inference(alpha=0.05)
	>>> ti.plot()
	'''
	YA,YB    = _datachecks.asmatrix(YA), _datachecks.asmatrix(YB)
	_datachecks.check('ttest_paired', YA, YB)
	return ttest(YA-YB)



def ttest2(YA, YB, equal_var=False):
	'''
	Two-sample t test.
	
	:Parameters:
	
	- *YA* --- (J x Q) data array  (J responses, Q nodes)
	- *YB* --- (J x Q) data array  (J responses, Q nodes)
	- *equal_var* --- If *True*, equal group variance will be assumed
	
	:Returns:
	
	- An **spm1d._spm.SPM_T** object.
	
	:Example:
	
	>>> YA,YB  = np.random.randn(8, 101), np.random.randn(8, 101)
	>>> YA,YB  = spm1d.util.smooth(Y, fwhm=10), spm1d.util.smooth(Y, fwhm=10)
	
	>>> t  = spm1d.stats.ttest2(YA, YB)
	>>> ti = t.inference(alpha=0.05)
	>>> ti.plot()
	'''
	### check data:
	YA,YB    = _datachecks.asmatrix(YA), _datachecks.asmatrix(YB)
	_datachecks.check('ttest2', YA, YB)
	### assemble data
	JA,JB    = YA.shape[0], YB.shape[0]
	Y        = np.vstack(  (YA, YB)  )
	### specify design and contrast:
	X        = np.zeros( (JA+JB, 2) )
	X[:JA,0] = 1
	X[JA:,1] = 1
	c        = (1, -1)
	### non-sphericity:
	Q        = None
	if not equal_var:
		J           = JA + JB
		q0,q1       = np.eye(JA), np.eye(JB)
		Q0,Q1       = np.matrix(np.zeros((J,J))), np.matrix(np.zeros((J,J)))
		Q0[:JA,:JA] = q0
		Q1[JA:,JA:] = q1
		Q           = [Q0, Q1]
	### compute SPM{t}:
	return glm(Y, X, c, Q)








import spm1d




#(0) Load dataset:
dataset = spm1d.data.mv0d.hotellings1.RSXLHotellings1()
# dataset = spm1d.data.mv0d.hotellings1.Sweat()
y,mu    = dataset.Y, dataset.mu
print dataset


#(1) Conduct T2 test:
T2      = spm1d.stats.hotellings(y, mu=mu)
T2i     = T2.inference(alpha=0.05)
print T2i







import numpy as np
from .. import _base




class RSWeightReduction(_base.DatasetT1):
	def _set_values(self):
		self.www  = 'http://www.real-statistics.com/students-t-distribution/one-sample-t-test/'
		self.Y    = np.array([23, 15, -5, 7, 1, -10, 12, -8, 20, 8, -2, -5])
		self.mu   = 0
		self.z    = 1.449255
		self.df   = 1,11
		self.p    = 0.087585


class ColumbiaSalmonella(_base.DatasetT1):
	def _set_values(self):
		self.www  = 'http://www.stat.columbia.edu/~martin/W2024/R2.pdf'
		self.Y    = np.array([0.593, 0.142, 0.329, 0.691, 0.231, 0.793, 0.519, 0.392, 0.418])
		self.mu   = 0.3
		self.z    = 2.2051
		self.df   = 1,8
		self.p    = 0.02927
	
	
	















from math import sqrt,log
import numpy as np
from scipy import ndimage
import _spm
import rft1d

eps        = np.finfo(float).eps   #smallest float, used to avoid divide-by-zero errors








def _fwhm(R):
	nComp  = R.shape[2]
	W      = [rft1d.geom.estimate_fwhm(R[:,:,i])  for i in range(nComp)]
	return np.mean(W)

	
def _get_residuals_onesample(Y):
	N = Y.shape[0]
	m = Y.mean(axis=0)
	R = Y - np.array([m]*N)
	return R

def _get_residuals_twosample(YA, YB):
	RA = _get_residuals_onesample(YA)
	RB = _get_residuals_onesample(YB)
	return np.vstack((RA,RB))

def _get_residuals_regression(y, x):
	J,Q,I      = y.shape  #nResponses, nNodes, nComponents
	Z          = np.matrix(np.ones(J)).T
	X          = np.hstack([np.matrix(x.T).T, Z])
	Xi         = np.linalg.pinv(X)
	R          = np.zeros(y.shape)
	for i in range(Q):
		for ii in range(I):
			yy     = np.matrix(y[:,i,ii]).T
			b      = Xi*yy
			eij    = yy - X*b
			R[:,i,ii] = np.asarray(eij).flatten()
	return R

def _get_residuals_manova1(Y, GROUP):
	u  = np.unique(GROUP)
	R  = []
	for uu in u:
		R.append(   _get_residuals_onesample(Y[GROUP==uu])   )
	return np.vstack(R)


def _normalize_residuals(R):
	nCurves,nNodes,nVectDim = R.shape
	for i in range(nCurves):
		mag       = np.sqrt(  (R[i,:,:]**2).sum(axis=0)  )   #normalize vector at this time node
		R[i,:,:] /= mag
		mag       = np.sqrt(  (R[i,:,:]**2).sum(axis=1)  )   #normalize the time series energy
		R[i,:,:] /= np.vstack([mag]*nVectDim).T
	return R


def _resel_counts(R, W):
	### construct binary search volume:
	a            = np.any(np.any(np.abs(R)>0, axis=0), axis=1)  #False indicates no observations at that node
	### summarize search area geometry:
	mNodes       = a.sum()
	mClusters    = ndimage.label(a)[1]
	### define resel counts:
	rCounts      = []
	rCounts.append(  mClusters  )
	rCounts.append( (mNodes-mClusters)/float(W) )   #number of non-zero nodes, normalized by FWHM
	return rCounts
	











import numpy as np
from matplotlib import pyplot
import spm1d






#(0) Load data:
dataset     = spm1d.data.uv0d.anova3onerm.NYUCaffeine()
dataset    = spm1d.data.uv0d.anova3onerm.Southampton3onerm()
y,A,B,C,SUBJ = dataset.get_data()
print dataset



#(1) Conduct ANOVA:
FF        = spm1d.stats.anova3onerm(y, A, B, C, SUBJ)
FFi       = [F.inference(0.05)  for F in FF]
fvalues   = [Fi.z  for Fi in FFi]
df        = [F.df  for F in FF]
pvalues   = [Fi.p  for Fi in FFi]
print 'Calculated results:'
print fvalues
print df
print pvalues


#
# #(2) Plot design:
# pyplot.close('all')
# design    = spm1d.stats.anova.designs.ANOVA3tworm(A, B, C, SUBJ)
# design.plot()
# pyplot.show()











# coding: utf-8

import os
import numpy as np
from .. import _base




class Neptune1999kneekin(_base.DatasetHotellingsPaired, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Neptune, R. R., Wright, I. C., & van den Bogert, A. J. (1999). Muscle coordination and function during cutting movements. Medicine & Science in Sports & Exercise, 31(2), 294–302.'
		self.www      = 'http://isbweb.org/data/rrn/index.html'
		self.note     = 'Results  ', 'Pataky, T., Robinson, M., & Vanrenterghem, J. (2015). Journal of Biomechanics. Journal of Biomechanics, 48(1), 190–192. http://doi.org/10.1016/j.jbiomech.2014.09.025'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Neptune1999kneekin.npz')
		Z             = np.load(self.datafile)
		self.YA       = Z['YA']
		self.YB       = Z['YB']
		Z.close()
		self.z        = None
		self.df       = None
		self.p        = None


class Pataky2014cop(_base.DatasetHotellingsPaired, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Robinson, M. A., Vanrenterghem, J., Savage, R., Bates, K. T., & Crompton, R. H. (2014). Vector field statistics for objective center-of-pressure trajectory analysis during gait, with evidence of scalar sensitivity to small coordinate system rotations. Gait and Posture, 1–4. http://doi.org/10.1016/j.gaitpost.2014.01.023'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Pataky2014cop.npz')
		Z             = np.load(self.datafile)
		self.YA       = Z['YA']
		self.YB       = Z['YB']
		Z.close()
		self.z        = None
		self.df       = None
		self.p        = None






import numpy as np
from matplotlib import pyplot
import spm1d


#(0) Load dataset:
dataset    = spm1d.data.uv1d.regress.SpeedGRF()
Y,x        = dataset.get_data()
# specify design matrix:
nCurves    = x.size
nFactors   = 4
X          = np.zeros((nCurves,nFactors))
X[:,0]     = x       #speed; the variable of interest
X[:,1]     = 1       #intercept
X[:,2]     = np.linspace(0,1,nCurves)   #linear drift
X[:,3]     = np.sin(np.linspace(0,np.pi,nCurves))  #sinusoidal drift
# specify a contrast vector:
c          = [1,0,0,0]  #speed (not the three other factors)



#(1) Conduct general linear test:
alpha      = 0.05
t          = spm1d.stats.glm(Y, X, c)
ti         = t.inference(alpha, two_tailed=False, interp=True)



#(2) Plot:
pyplot.close('all')
ax     = pyplot.axes()
ti.plot()
ti.plot_threshold_label(fontsize=8)
ti.plot_p_values(size=10, offsets=[(0,0.3)])
ax.set_xlabel('Time (%)')
pyplot.show()



import numpy as np
from .. import _base




class RSWeightClinic(_base.DatasetTpaired):
	def _set_values(self):
		self.www  = 'http://www.real-statistics.com/students-t-distribution/paired-sample-t-test/'
		self.YA   = np.array([210,205,193,182,259,239,164,197,222,211,187,175,186,243,246])
		self.YB   = np.array([197,195,191,174,236,226,157,196,201,196,181,164,181,229,231])
		# self.Y    = np.hstack([self.YA, self.YB])
		# self.A    = np.array([0]*15 + [1]*15)
		self.z    = 6.6896995
		self.df   = 1,14
		self.p    = 1.028e-5



class ColumbiaMileage(_base.DatasetTpaired):
	def _set_values(self):
		self.www  = 'http://www.stat.columbia.edu/~martin/W2024/R2.pdf'
		self.YA   = np.array([19, 22, 24, 24, 25, 25, 26, 26, 28, 32])
		self.YB   = np.array([16, 20, 21, 22, 23, 22, 27, 25, 27, 28])
		# self.Y    = np.hstack([self.YA, self.YB])
		# self.A    = np.array([0]*15 + [1]*15)
		self.z    = 4.4721
		self.df   = 1,9
		self.p    = 0.000775
















import numpy as np
from matplotlib import pyplot
import spm1d




#(0) Load data:
dataset    = spm1d.data.uv0d.anova2onerm.Santa23()
# dataset    = spm1d.data.uv0d.anova2onerm.Southampton2onerm()
dataset    = spm1d.data.uv0d.anova2onerm.RSXLDrug()
y,A,B,SUBJ = dataset.Y, dataset.A, dataset.B, dataset.SUBJ
print dataset




#(1) Conduct ANOVA:
FF        = spm1d.stats.anova2onerm(y, A, B, SUBJ)
FFi       = [F.inference(0.05)  for F in FF]
fvalues   = [F.z   for F in FF]
df        = [F.df  for F in FF]
pvalues   = [Fi.p  for Fi in FFi]
print 'Calculated results:'
print fvalues
print df
print pvalues

# print FF



#(2) Plot design:
pyplot.close('all')
design    = spm1d.stats.anova.designs.ANOVA2onerm(A, B, SUBJ)
design.plot()
pyplot.show()





# coding: utf-8

import os
import numpy as np
from .. import _base





class Random(_base.DatasetT1, _base.Dataset1D):
	def _set_values(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'random.npy')
		self.Y        = np.load(self.datafile)
		self.mu       = 0
		self.z        = None
		self.df       = None
		self.p        = None


class SimulatedPataky2015a(_base.DatasetT1, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Vanrenterghem, J., & Robinson, M. A. (2015). Zero- vs. one-dimensional, parametric vs. non-parametric, and confidence interval vs. hypothesis testing procedures in one-dimensional biomechanical trajectory analysis. Journal of Biomechanics, 1–9. http://doi.org/10.1016/j.jbiomech.2015.02.051'
		self.datafile = os.path.join(_base.get_datafilepath(), 'simscalar_datasetA.npy')
		self.Y        = np.load(self.datafile)
		self.mu       = 0
		self.z        = None
		self.df       = None
		self.p        = None

class SimulatedPataky2015b(_base.DatasetT1, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Vanrenterghem, J., & Robinson, M. A. (2015). Zero- vs. one-dimensional, parametric vs. non-parametric, and confidence interval vs. hypothesis testing procedures in one-dimensional biomechanical trajectory analysis. Journal of Biomechanics, 1–9. http://doi.org/10.1016/j.jbiomech.2015.02.051'
		self.datafile = os.path.join(_base.get_datafilepath(), 'simscalar_datasetB.npy')
		self.Y        = np.load(self.datafile)
		self.mu       = 0
		self.z        = None
		self.df       = None
		self.p        = None










import numpy as np
from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset    = spm1d.data.uv0d.anova2nested.QIMacros()
dataset    = spm1d.data.uv0d.anova2nested.SouthamptonNested1()
y,A,B      = dataset.Y, dataset.A, dataset.B
# print dataset



#(1) Conduct ANOVA:
FF        = spm1d.stats.anova2nested(y, A, B, equal_var=True)
# FFi       = [F.inference(0.05)  for F in FF]
# fvalues   = [Fi.z  for Fi in FFi]
# df        = [F.df  for F in FF]
# pvalues   = [Fi.p  for Fi in FFi]
# print 'Calculated results:'
# print fvalues
# print df
# print pvalues
#
#
#
# #(2) Plot design:
# pyplot.close('all')
# design    = spm1d.stats.anova.designs.ANOVA2nested(A, B)
# design.plot()
# pyplot.show()



# coding: utf-8

import os
import numpy as np
from .. import _base




class SpeedGRFcategoricalRM(_base.DatasetANOVA1rm, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Caravaggi, P., Savage, R., Parker, D., Goulermas, J., Sellers, W., & Crompton, R. (2008). New insights into the plantar pressure correlates of walking speed using pedobarographic statistical parametric mapping (pSPM). Journal of Biomechanics, 41(9), 1987–1994.'
		self.datafile = os.path.join(_base.get_datafilepath(), 'ex_grf_means.npz')
		Z             = np.load(self.datafile)
		self.Y        = Z['Y']
		self.A        = Z['SPEED']
		self.SUBJ     = Z['SUBJ']
		Z.close()
		self.z        = None
		self.df       = None
		self.p        = None









import numpy as np
import rft1d
from .. import _spm



eps         = np.finfo(float).eps




class _ANOVA(object):
	def __init__(self, Y, X, contrasts):
		Y               = np.asarray(Y, dtype=float)
		self.dim        = Y.ndim - 1            #dependent variable dimensionality (0 or 1)
		self.Y          = self._asmatrix(Y)     #stacked dependent variable (JxQ)
		self.X          = np.matrix(X)          #design matrix
		self.J          = self.X.shape[0]       #number of observations
		self.Q          = self.Y.shape[1]       #number of field nodes
		self.eij        = None
		self.contrasts  = contrasts             #list of contrast objects
		self._R         = None                  #residual forming matrix
		self._beta      = None                  #least-squares parameters
		self._rankR     = None                  #error degrees of freedom
		self._dfE       = None                  #error degrees of freedom (equivalent to _rankR)
		self._SSE       = None                  #sum-of-squares (error)
		self._MSE       = None                  #mean-squared error
		if self.dim==1:
			self.eij    = None
			self.fwhm   = None
			self.resels = None
		self._estimate_parameters()

	
	### PRIVATE METHODS ##################################
		
	def _asmatrix(self, Y):
		return np.matrix(Y).T if Y.ndim==1 else np.matrix(Y)
		
	def _estimate_parameters(self):
		Y,X,J           = self.Y, self.X, self.J
		Xi              = np.linalg.pinv(X)         #design matrix pseudoinverse
		self._beta      = Xi*Y                      #estimated parameters
		self._R         = np.eye(J) - X*Xi          #residual forming matrix
		self._rankR     = self._rank(self._R)
		self._SSE       = np.diag( Y.T * self._R * Y )
		self._dfE       = self._rankR
		self.eij        = np.asarray(self.Y - X*self._beta)  #residuals
		if self._dfE > eps:
			self._MSE = self._SSE / self._dfE
		if self.dim==1:
			self.fwhm   = rft1d.geom.estimate_fwhm(self.eij)            #smoothness
			self.resels = rft1d.geom.resel_counts(self.eij, self.fwhm, element_based=False) #resel 
			
	# def _rank(self, A, tol=None):
	# 	return float( np.linalg.matrix_rank(A, tol=tol) )

	def _rank(self, A, tol=None):
		'''
		This is a slight modification of np.linalg.matrix_rank.
		The tolerance performs poorly for some matrices
		Here the tolerance is boosted by a factor of ten for improved performance.
		'''
		M = np.asarray(A)
		S = np.linalg.svd(M, compute_uv=False)
		if tol is None:
			tol = 10 * S.max() * max(M.shape) * np.finfo(M.dtype).eps
		rank = sum(S > tol)
		return rank



	### COMPUTATIONAL CORE (PRIVATE) ##################################
	###  all methods accept contrast matrices (as np.matrix)

	def _compute_ss(self, C):
		J,Y,X,b,R     = self.J, self.Y, self.X, self._beta, self._R
		X0            = self._get_design_matrix_reduced(C)    #reduced design matrix
		R0            = np.eye(J) - X0*np.linalg.pinv(X0)     #residual forming matrix (reduced design)
		M             = R0 - R                                #projection matrix
		SS            = np.diag( b.T * X.T * M * X * b )
		df            = self._rank(M)
		if self.dim == 0:
			return float(SS), df
		else:
			return SS, df

	def _compute_ss_rm(self, C):
		J,Y,X,b,R     = self.J, self.Y, self.X, self._beta, self._R
		X0            = self._get_design_matrix_reduced(C)    #reduced design matrix
		R0            = np.eye(J) - X0*np.linalg.pinv(X0)     #residual forming matrix (reduced design)
		M             = R0 - R                                #projection matrix
		Xb            = X*b
		SS            = np.diag( b.T * X.T * M * Xb )
		eij           = np.asarray(Y - X0*b)  #residuals
		df            = self._rank(M)
		if self.dim == 0:
			return float(SS), df, eij, X0
		else:
			return SS, df, eij, X0

	def _compute_test_statistic(self, C):
		SS,df         = self._compute_ss(C)
		MS            = SS / df
		f             = MS / self._MSE
		return f, (df,self._dfE), (SS,self._SSE), (MS,self._MSE)
	
	def _compute_test_statistic_rm(self, contrast):
		C,CE          = contrast.get_compound_matrices()
		SS0,df0,eij0,X00  = self._compute_ss_rm(C.T)
		SSE,dfE,eij,X0    = self._compute_ss_rm(CE.T)
		SS            = SS0 - SSE
		df            = df0 - dfE
		if df<0:  #df will be less than zero if a redundant term is added to the contrast when there are more than 4 factor levels;  see factors._00_parse. if n>4
			df        = contrast.n
		MS,MSE        = (SS/df), (SSE/dfE)
		f             = MS / MSE
		return f, (df,dfE), (SS,SSE), (MS,MSE), eij, X0

	def _get_design_matrix_reduced(self, C):
		# Xc   = self.X * C
		C0   = np.eye(C.shape[0]) - C*np.linalg.pinv(C)
		X0   = self.X * C0
		return X0


	### PUBLIC METHODS ##################################
	###  all methods accept contrast objects
	
	def compute_ss(self, contrast):
		return self._compute_ss(contrast.C.T)
	def compute_test_statistic(self, contrast):
		return self._compute_test_statistic(contrast.C.T)
	def compute_test_statistics(self):
		F = []
		for contrast in self.contrasts:
			if contrast.isrm:
				f,df,ss,ms,eij,X0  = self._compute_test_statistic_rm(contrast)
			else:
				f,df,ss,ms  = self._compute_test_statistic(contrast.C.T)
				eij         = self.eij
				X0          = None
			if self.dim == 0:
				F.append( _spm.SPM0D_F(f, df, ss, ms, eij, X0) )
			else:
				F.append( _spm.SPM_F(f, df, self.fwhm, self.resels, self.X, self._beta, eij, X0) )
		return F
	def get_design_matrix_reduced(self, contrast):
		return self._get_design_matrix_reduced(contrast.C.T)
	




class ANOVA1(_ANOVA):
	pass
class ANOVA1rm(_ANOVA):
	pass
class ANOVA2(_ANOVA):
	pass
class ANOVA2rm(_ANOVA):
	pass
class ANOVA2onerm(_ANOVA):
	pass
class ANOVA3(_ANOVA):
	pass
class ANOVA3onerm(_ANOVA):
	pass
class ANOVA3tworm(_ANOVA):
	pass




class ANOVA2nested(_ANOVA):
	def compute_test_statistics(self):
		CA,CB         = [c.C for c in self.contrasts]
		SSA,uA        = self._compute_ss(CA.T)
		SSB,uB        = self._compute_ss(CB.T)
		MSA,MSB       = (SSA/uA), (SSB/uB)
		fA            = MSA / MSB
		fB            = MSB / self._MSE
		### assemble:
		SS,MS,DF      = [(SSA,SSB), (SSB,self._SSE)], [(MSA,MSB), (MSB,self._MSE)], [(uA,uB), (uB,self._dfE)]
		F             = [float(fA), float(fB)] if self.dim==0 else [fA,fB]
		return [_spm.SPM0D_F(f, df, ss, ms)  for f,df,ss,ms in zip(F,DF,SS,MS)]






class ANOVA3nested(_ANOVA):
	def compute_test_statistics(self):
		CA,CB,CC      = [c.C for c in self.contrasts]
		SSA,uA        = self._compute_ss(CA.T)
		SSB,uB        = self._compute_ss(CB.T)
		SSC,uC        = self._compute_ss(CC.T)
		MSA,MSB,MSC   = (SSA/uA), (SSB/uB), (SSC/uC)
		fA,fB,fC      = MSA/MSB, MSB/MSC, MSC/self._MSE
		### assemble:
		F             = [fA, fB, fC]
		DF            = [(uA,uB), (uB,uC), (uC,self._dfE)]
		SS            = [(SSA,SSB), (SSB,SSC), (SSC,self._SSE)]
		MS            = [(MSA,MSB), (MSB,MSC), (MSC,self._MSE)]
		F             = map(float, F) if self.dim==0 else F
		return [_spm.SPM0D_F(f, df, ss, ms)  for f,df,ss,ms in zip(F,DF,SS,MS)]






import numpy as np
from matplotlib import pyplot
import spm1d



#(0) Load dataset:
dataset    = spm1d.data.uv1d.regress.SimulatedPataky2015c()
# dataset    = spm1d.data.uv1d.regress.SpeedGRF()
Y,x        = dataset.get_data()



#(1) Conduct regression:
alpha      = 0.05
t          = spm1d.stats.regress(Y, x)
ti         = t.inference(alpha, two_tailed=False, interp=True)



#(2) Plot:
pyplot.close('all')
ax     = pyplot.axes()
ti.plot()
ti.plot_threshold_label(fontsize=8)
ti.plot_p_values(size=10, offsets=[(0,0.3)])
ax.set_xlabel('Time (%)')
pyplot.show()






import numpy as np
from matplotlib import pyplot
import spm1d





#(0) Load data:
dataset   = spm1d.data.uv0d.anova3.RSItalian()
# dataset   = spm1d.data.uv0d.anova3.SouthamptonFullyCrossedMixed()
y,A,B,C   = dataset.get_data()
print dataset



#(1) Conduct ANOVA:
FF        = spm1d.stats.anova3(y, A, B, C, equal_var=False)
FFi       = [F.inference(0.05)  for F in FF]
fvalues   = [Fi.z  for Fi in FFi]
df        = [F.df  for F in FF]
pvalues   = [Fi.p  for Fi in FFi]
print 'Calculated results:'
print fvalues
print df
print pvalues



#(2) Plot design:
pyplot.close('all')
design    = spm1d.stats.anova.designs.ANOVA3(A, B, C)
design.plot()
pyplot.show()






import numpy as np


def flatten0d(Y):
	y   = np.hstack(Y.T)
	A   = np.hstack(  [ [i]*Y.shape[0] for i in range(Y.shape[1]) ]  )
	return y,A
	
def flatten0d_rm(Y):
	y    = np.hstack(Y.T)
	# A    = np.hstack(  [ [i]*Y.shape[0] for i in range(Y.shape[1]) ]  )
	A    = np.hstack(  range(Y.shape[1])*Y.shape[0]  )
	SUBJ = np.hstack(  [ [i]*Y.shape[1] for i in range(Y.shape[0]) ]  )
	return y,A,SUBJ

def flatten0d_ub(Y):
	A   = np.hstack(  [ [i]*y.size for i,y in enumerate(Y) ]  )
	y   = np.hstack(Y.T)
	return y,A


def stack1d(YY):
	Y   = np.vstack(YY)
	# nA  = len(YY)
	# nnA = [len(y) for y in YY]
	A   = np.hstack(  [ [i]*len(y) for i,y in enumerate(YY) ]  )
	return Y,A
	# self.GROUP    = np.array([[i]*j for i,j in enumerate(self.JJ)]).flatten()
	
def stack1d_rm(YY):
	nCond = len(YY)
	nSubj = YY[0].shape[0]
	Y     = np.vstack(YY)
	A     = np.hstack(  [ [i]*nSubj for i in range(nCond) ]  )
	SUBJ  = np.hstack(  range(nSubj) * nCond  )
	return Y,A,SUBJ
	








import numpy as np
from matplotlib import pyplot
import spm1d





#(0) Load data:
dataset   = spm1d.data.uv0d.anova2.Mouse()       #2x2
dataset   = spm1d.data.uv0d.anova2.Detergent()     #2x3
dataset   = spm1d.data.uv0d.anova2.Satisfaction()  #2x3
dataset   = spm1d.data.uv0d.anova2.SouthamptonCrossed1()  #2x3
y,A,B     = dataset.get_data()
# print dataset




#(1) Conduct ANOVA:
FF        = spm1d.stats.anova2(y, A, B, equal_var=False)
FFi       = [F.inference(0.05)  for F in FF]
fvalues   = [Fi.z  for Fi in FFi]
df        = [F.df  for F in FF]
pvalues   = [Fi.p  for Fi in FFi]
print 'Calculated results:'
print fvalues
print df
print pvalues



#(2) Plot design:
pyplot.close('all')
design    = spm1d.stats.anova.designs.ANOVA2(A, B)
design.plot()
pyplot.show()




import numpy as np
from scipy import stats
from matplotlib import pyplot
import spm1d




#(0) Example one-way (0D):
dataset = spm1d.data.uv0d.anova1.Cars()
# dataset = spm1d.data.uv0d.anova1.Sound()
# dataset = spm1d.data.uv0d.anova1.Southampton1()
# dataset = spm1d.data.uv0d.anova1.ConstructionUnequalSampleSizes()
# dataset = spm1d.data.uv0d.anova1.RSUnequalSampleSizes()
print dataset
y,A     = dataset.get_data()



#(1) Conduct ANOVA:
spmF  = spm1d.stats.anova1(y, A, equal_var=True)
spmFi   = spmF.inference(0.05)
print spmFi


#(2) Compare to scipy.stats result:
yy      = [y[A==u] for u in np.unique(A)]
f,p     = stats.f_oneway(*yy)
print 'scipy.stats result:\n   F = %.5f\n   p = %.5f' %(f,p)



#(3) Plot design:
pyplot.close('all')
design    = spm1d.stats.anova.designs.ANOVA1(A)
design.plot()
pyplot.show()



import numpy as np
from .. import _base



class Cars(_base.DatasetANOVA1):
	def _set_values(self):
		self.www     = 'http://cba.ualr.edu/smartstat/topics/anova/example.pdf'
		Y  = np.array([
			[643, 469, 484],
			[655, 427, 456],
			[702, 525, 402],
		])
		self.Y     = Y.T.flatten()
		self.A     = np.hstack( [0]*3 + [1]*3 + [2]*3 )
		self.z     = 25.17
		self.df    = (2, 6)
		self.p     = 0.001207


class ConstructionUnequalSampleSizes(_base.DatasetANOVA1):
	def _set_values(self):
		self.www     = 'http://stackoverflow.com/questions/8320603/how-to-do-one-way-anova-in-r-with-unequal-sample-sizes'
		Y  = np.array([
			[34,25,27,31,26,34,21],
			[33,35,31,31,42,33],
			[17,30,30,26,32,28,26,29],
			[28,33,31,27,32,33,40],
		])
		self.Y     = np.hstack(Y)
		self.A     = np.hstack([[i]*len(yy) for i,yy in enumerate(Y)])
		self.z     = 3.4971
		self.df    = (3, 24)
		self.p     = 0.03098



class RSUnequalSampleSizes(_base.DatasetANOVA1):
	def _set_values(self):
		self.www     = 'http://www.real-statistics.com/one-way-analysis-of-variance-anova/unplanned-comparisons/anova-unequal-sample-sizes/'
		Y  = np.array([
			[3, 5, 6, 1, 5, 6, 4, 3, 7, 4, 5],
			[2, 4, 3, 5, 1, 5, 2, 3, 6, 2, 4, 1],
			[5, 8, 6, 4, 7, 8, 5, 6, 6],
			[4, 5, 3, 7, 6, 3, 2, 2, 4, 5],
		])
		self.Y     = np.hstack(Y)
		self.A     = np.hstack([[i]*len(yy) for i,yy in enumerate(Y)])
		self.z     = 5.864845
		self.df    = (3, 38)
		self.p     = 0.00215




class Sound(_base.DatasetANOVA1):
	def _set_values(self):
		self.www     = 'http://web.mst.edu/~psyworld/anovaexample.htm'
		Y  = np.array([
			[7, 4, 6, 8, 6, 6, 2, 9],
			[5, 5, 3, 4, 4, 7, 2, 2],
			[2, 4, 7, 1, 2, 1, 5, 5],
		]).T
		self.Y     = Y.T.flatten()
		self.A     = np.hstack( [0]*8 + [1]*8 + [2]*8 )
		self.z     = 3.59
		self.df    = (2, 21)
		self.p     = 0.0456    #calculated as:  scipy.stats.f.sf(3.59, 2, 21)


class Southampton1(_base.DatasetANOVA1):
	def _set_values(self):
		self.www     = 'http://www.southampton.ac.uk/~cpd/anovas/datasets/Doncaster&Davey%20-%20Model%201_1%20One%20factor.txt'
		self.Y       = np.array([4.5924, -0.5488, 6.1605, 2.3374, 5.1873, 3.3579, 6.3092, 3.2831, 7.3809, 9.2085, 13.1147, 15.2654, 12.4188, 14.3951, 8.5986, 3.4945, 21.3220, 25.0426, 22.6600, 24.1283, 16.5927, 10.2129, 9.8934, 10.0203])
		self.A       = np.array([1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3])
		self.z       = 17.08
		self.df      = (2, 21)
		self.p       = '<0.001'








import numpy as np
from matplotlib import pyplot
import spm1d






#(0) Load data:
dataset      = spm1d.data.uv1d.anova1.SpeedGRFcategorical()
Y,A          = dataset.get_data()
Y1,Y2,Y3     = [Y[A==u] for u in np.unique(A)]


#(1) Conduct Post hoc tests:
alpha        = 0.05
nTests       = 3
p_critical   = spm1d.util.p_critical_bonf(alpha, nTests)
### t statistics:
t12   = spm1d.stats.ttest2(Y1, Y2, equal_var=False)
t13   = spm1d.stats.ttest2(Y1, Y3, equal_var=False)
t23   = spm1d.stats.ttest2(Y2, Y3, equal_var=False)
### inference:
t12i  = t12.inference(alpha=p_critical, two_tailed=True)
t13i  = t13.inference(alpha=p_critical, two_tailed=True)
t23i  = t23.inference(alpha=p_critical, two_tailed=True)



#(2) Plot results:
pyplot.close('all')
Fi.plot()
Fi.plot_threshold_label(bbox=dict(facecolor='w'))
pyplot.ylim(-1, 500)
pyplot.xlabel('Time (%)', size=20)
pyplot.title(r'Critical threshold at $\alpha$=%.2f:  $F^*$=%.3f' %(alpha, Fi.zstar))
pyplot.show()










import numpy as np
from matplotlib import pyplot




class Factor(object):
	def __init__(self, A):
		self.A          = np.asarray(A, dtype=int)        #integer vector of factor levels
		self.J          = None     #number of observations
		self.u          = None     #unique levels
		self.n          = None     #number of levels
		self.pairs      = None     #pairs of factor levels
		self.nPairs     = None     #number of factor-level pairs
		self.isnested   = False    #nested flag
		self.isrm       = False    #repeated measures flag
		self.nested     = None
		self.balanced   = True
		self._00_parse()
		self._01_check_unbalanced()

	def _00_parse(self):
		self.J            = self.A.size
		self.u            = np.unique(self.A)
		self.n            = self.u.size
		self.pairs        = np.vstack([self.u[:-1], self.u[1:]]).T
		if self.n>4:  ### add a redundant term
			self.pairs    = np.vstack([self.pairs, self.u[[-1,0]] ])
		self.nPairs       = self.pairs.shape[0]
		
	def _01_check_unbalanced(self):
		nn  = [(self.A==uu).sum()   for uu in self.u]
		self.balanced = np.all( nn==nn[0] )


	def check_balanced(self, other):
		A,B = self.A, other.A
		N   = []
		for uA in self.u:
			for uB in other.u:
				N.append( ((A==uA) & (B==uB)).sum()  )
		return np.all( N==N[0] )


	def check_balanced_nested(self, other):
		A,B   = self.A, other.A
		N     = []
		for uA,UB in zip(self.u,other.u):
			for uB in UB:
				N.append( ((A==uA) & (B==uB)).sum()  )
		return np.all( N==N[0] )


	def check_balanced_nested3(self, other, another):
		A,B,C = self.A, other.A, another.A
		N     = []
		for uA,UB,UUC in zip(self.u,other.u,another.u):
			for uB,UC in zip(UB, UUC):
				for uC in UC:
					N.append( ((A==uA) & (B==uB) & (C==uC) ).sum()  )
		return np.all( N==N[0] )


	def check_balanced_rm(self, other):
		S,A = self.A, other.A
		N   = []
		for uS in self.u:
			UA  = np.unique(A[S==uS])
			for uA in UA:
				N.append( ((S==uS) & (A==uA)).sum()  )
		return np.all( N==N[0] )



	def get_Q(self):
		return [np.matrix(np.diag(self.A==u), dtype=float) for u in self.u]
	
	def get_design_interaction(self, other):
		XAB        = []
		A,B        = self.A, other.A
		for uA0,uA1 in self.pairs:
			for uB0,uB1 in other.pairs:
				x      = np.zeros(self.J)
				x[(A==uA0)&(B==uB0)] =  1
				x[(A==uA0)&(B==uB1)] = -1
				x[(A==uA1)&(B==uB0)] = -1
				x[(A==uA1)&(B==uB1)] =  1
				XAB.append(x)
		return np.matrix(XAB).T




	
	def get_design_interaction_3way(self, other, another):
		XABC       = []
		A,B,C      = self.A, other.A, another.A
		pA,pB,pC   = self.pairs, other.pairs, another.pairs
		for uA0,uA1 in pA:
			for uB0,uB1 in pB:
				for uC0,uC1 in pC:
					x      = np.zeros(self.J)
					### part one:
					x[(A==uA0)&(B==uB0)&(C==uC0)] =  1
					x[(A==uA0)&(B==uB0)&(C==uC1)] = -1
					x[(A==uA0)&(B==uB1)&(C==uC0)] = -1
					x[(A==uA0)&(B==uB1)&(C==uC1)] =  1
					### part two:
					x[(A==uA1)&(B==uB0)&(C==uC0)] = -1
					x[(A==uA1)&(B==uB0)&(C==uC1)] =  1
					x[(A==uA1)&(B==uB1)&(C==uC0)] =  1
					x[(A==uA1)&(B==uB1)&(C==uC1)] = -1
					XABC.append(x)
		return np.matrix(XABC).T

	def get_design_main(self, simplified=False):
		X = []
		if simplified:
			for u in self.u:
				x     = np.zeros(self.J)
				x[self.A==u] = 1
				X.append(x)
		else:
			for u0,u1 in self.pairs:
				x        = np.zeros(self.J)
				x[self.A==u0] =  1
				x[self.A==u1] = -1
				X.append(x)
		return np.matrix( X ).T




class FactorSubject(Factor):
	def __init__(self, S):
		super(FactorSubject, self).__init__(S)




class FactorRM(Factor):
	def __init__(self, A, S):
		super(FactorRM, self).__init__(A)
		self.S     = S
		self.isrm  = True

	def get_design_subject_pooled(self):
		return self.S.get_design_main(simplified=True)
		
	def get_design_subject_partitioned(self):
		XS        = []
		for u0,u1 in self.pairs:
			for u in self.S.u:
				x    = np.zeros(self.J)
				x[(self.A==u0) & (self.S.A==u)] = +1
				x[(self.A==u1) & (self.S.A==u)] = -1
				XS.append(x)
		return np.matrix(XS).T


	def get_design_subject_partitioned3(self, other):
		XS        = []
		B,C       = self.A, other.A
		for uB0,uB1 in self.pairs:
			for uC0,uC1 in other.pairs:
				for u in self.S.u:
					x    = np.zeros(self.J)
					x[(B==uB0) & (C==uC0) & (self.S.A==u)]  = +1
					x[(B==uB0) & (C==uC1) & (self.S.A==u)]  = -1
					x[(B==uB1) & (C==uC0) & (self.S.A==u)]  = -1
					x[(B==uB1) & (C==uC1) & (self.S.A==u)]  = +1
					XS.append(x)
		return np.matrix(XS).T
			
			
	def get_design_subject_interaction(self, other):
		XS        = []
		A,B,S     = self.A, other.A, self.S.A
		for uA0,uA1 in self.pairs:
			for uB0,uB1 in other.pairs:
				for u in self.S.u:
					x    = np.zeros(self.J)
					x[(A==uA0)&(B==uB0) & (S==u)] = +1
					x[(A==uA0)&(B==uB1) & (S==u)] = -1
					x[(A==uA1)&(B==uB0) & (S==u)] = -1
					x[(A==uA1)&(B==uB1) & (S==u)] = +1
					XS.append(x)
		return np.matrix(XS).T





class FactorNested(Factor):
	def __init__(self, A, nestfactor):
		self.NEST       = nestfactor
		super(FactorNested, self).__init__(A)


	def _00_parse(self):
		self.J          = self.A.size
		self.u          = [np.unique(self.A[self.NEST.A==u])  for u in self.NEST.u]
		self.n          = [u.size for u in self.u]
		self.pairs      = [np.vstack([u[:-1], u[1:]]).T   for u in self.u]
		self.nPairs     = [pair.shape[0]  for pair in self.pairs]

	def _01_check_unbalanced(self):
		pass
		# nn  = [(self.A==uu).sum()   for uu in self.u]
		# self.balanced = np.all( nn==nn[0] )

	def get_Q(self):
		Q = []
		for u in self.u:
			for uu in u:
				q = np.matrix(np.diag(self.A==uu), dtype=float)
				Q.append(q)
		return Q
	
	
	def get_design_main(self):
		A,B   = self.NEST.A, self.A
		X     = []
		for u,pairs in zip(self.NEST.u, self.pairs):
			for u0,u1 in pairs:
				x        = np.zeros(self.J)
				x[(A==u)&(B==u0)] =  1
				x[(A==u)&(B==u1)] = -1
				X.append(x)
		return np.matrix( X ).T



class FactorNested2(Factor):
	def __init__(self, A, nestfactor):
		self.NEST0      = nestfactor.NEST
		self.NEST1      = nestfactor
		super(FactorNested2, self).__init__(A)


	def _00_parse(self):
		A,B,C           = self.NEST0.A, self.NEST1.A, self.A
		uA,UB           = self.NEST0.u, self.NEST1.u
		self.J          = self.A.size
		self.u          = [[np.unique(C[(A==uuA)&(B==uuB)]) for uuB in uB]  for uuA,uB in zip(uA, UB)]
		self.n          = [[uu.size for uu in u] for u in self.u]
		self.pairs      = [[np.vstack([uu[:-1], uu[1:]]).T   for uu in u] for u in self.u]
		self.nPairs     = [[p.shape[0]  for p in pair] for pair in self.pairs]

	def _01_check_unbalanced(self):
		pass

	def get_Q(self):
		Q = []
		for u in self.u:
			for uu in u:
				for uuu in uu:
					q = np.matrix(np.diag(self.A==uuu), dtype=float)
					Q.append(q)
		return Q
	
	def get_design_main(self):
		A,B,C = self.NEST0.A, self.NEST1.A, self.A
		X     = []
		for uA,UB,PAIRSC in zip(self.NEST0.u, self.NEST1.u, self.pairs):
			for uB,pairsC in zip(UB,PAIRSC):
				for u0,u1 in pairsC:
					x        = np.zeros(self.J)
					x[(A==uA)&(B==uB)&(C==u0)] =  1
					x[(A==uA)&(B==uB)&(C==u1)] =  -1
					X.append(x)
		return np.matrix( X ).T







'''
Input/Output module

This module contains functions for loading and saving data, including
SPMs.

.. warning:: most functions in **spm1d.io** are deprecated.

	Please use the following well-supported functions:
	
	* numpy.loadtxt
	* numpy.savetxt
	* numpy.load
	* numpy.save
	* scipy.io.loadmat
	* scipy.io.savemat
'''

# Copyright (C) 2014  Todd Pataky
# io.py version: 0.2 (2014/05/01)


import numpy as np
from scipy.io import loadmat as scipy_loadmat
from scipy.io import savemat as scipy_savemat
from stats import _spm



def _check4pytables():
	try:
		import tables
	except ImportError:
		raise(ImportError('You must install pytables to use spm1d.io.load'))
	



def load(fname):
	'''
	WARNING:  "spm1d.io.load" is deprecated and will be removed from future versions of spm1d. 
	'''
	_check4pytables()
	import tables
	fid  = tables.openFile(fname, mode='r')
	Y    = fid.getNode('/Y').read()
	fid.close()
	return Y


def loadmat(fname):
	'''
	WARNING:  "spm1d.io.loadmat" is deprecated.  Use "scipy.io.loadmat" instead. 
	'''
	raise DeprecationWarning('"spm1d.io.loadmat" is deprecated.  Use "scipy.io.loadmat" instead')


def loadspm(fname):
	'''
	Load an SPM object from disk.
	The SPM file should have been saved in zipped NumPy (NPZ) format using **spm1d.io.savespm**.
	
	:Parameters:
	
	fname : a string specifying the file name
	
	:Returns:
	
	An **spm1d** SPM object.
	'''
	Z           = np.load(fname)
	keys        = Z.keys()
	STAT        = str( Z['STAT'] )
	X           = Z['X']
	beta        = Z['beta']
	residuals   = Z['residuals']
	z           = Z['z']
	df          = tuple( Z['df'] )
	fwhm        = float( Z['fwhm'] )
	resels      = tuple( Z['resels'] )
	if 'alpha' in keys:
		alpha   = float( Z['alpha'] )
		zstar   = float( Z['zstar'] )
		p       = tuple( Z['p'] )
		nClusters = int( Z['nClusters'] )
		L       = Z['L']
		two_tailed = bool( Z['two_tailed'] )
	if (STAT=='F') and ('alpha' in keys):
		pass
	elif STAT=='F':
		spm     = _spm.SPM_F(z, df, fwhm, resels, X, beta, residuals)
		if 'alpha' in keys:
			spm = _spm.SPMi_F(spm, alpha, zstar, nClusters, L, p, two_tailed)
	elif STAT=='T':
		spm     = _spm.SPM_T(z, df, fwhm, resels, X, beta, residuals)
		if 'alpha' in keys:
			spm = _spm.SPMi_T(spm, alpha, zstar, nClusters, L, p, two_tailed)
	return spm


def loadtxt(fname):
	'''
	WARNING:  "spm1d.io.loadtxt" is deprecated.  Use "numpy.loadtxt" instead. 
	'''
	raise DeprecationWarning('"spm1d.io.loadtxt" is deprecated.  Use "numpy.loadtxt" instead.')
	return np.loadtxt(fname, delimiter='\t')


def save(fname, Y):
	'''
	WARNING:  "spm1d.io.save" is deprecated and will be removed from future versions of spm1d.
	'''
	_check4pytables()
	import tables
	### create file (existing file will be overwritten)
	fid          = tables.openFile(fname, mode='w')
	try:
		### write data:
		atom     = tables.FloatAtom()
		filter0  = tables.Filters(complevel=5,complib='zlib')
		CA       = fid.createCArray(fid.root, 'Y', atom, Y.shape, filters=filter0)
		CA[:]    = Y
		fid.close()
	except IOError:
		fid.close()
		raise IOError('Error saving file.')


def savemat(fname, Y):
	'''
	WARNING:  "spm1d.io.savemat" is deprecated.  Use "scipy.io.savemat" instead. 
	'''
	raise DeprecationWarning('"spm1d.io.savemat" is deprecated.  Use "scipy.io.savemat" instead.')



def savespm(fname, spm):
	'''
	Save an SPM object in zipped NumPy (NPZ) format. Once saved an SPM can be re-loaded using **spm1d.io.loadspm**
	
	:Parameters:
	
	fname : a string specifying the file name (with extension .npz)
	
	spm   : an SPM object, as returned from a statistical test in **spm1d.stats**
	'''
	if not isinstance(spm, (_spm._SPM, _spm._SPMinference)):
		raise IOError('Must submit an SPM or SPMi object to spm1d.io.save_spm')
	fields      = ['STAT', 'Q', 'X', 'z', 'df', 'fwhm', 'resels', 'beta', 'residuals']
	if isinstance(spm, _spm._SPMinference):
		fields += ['alpha', 'zstar', 'p', 'nClusters', 'L', 'two_tailed']
	values      = [eval('spm.%s' %field)  for field in fields]
	np.savez(fname, **dict(zip(fields,values)) )


def savetxt(fname, Y):
	'''
	WARNING:  "spm1d.io.savetxt" is deprecated.  Use "numpy.savetxt" instead. 
	'''
	raise DeprecationWarning('"spm1d.io.savetxt" is deprecated.  Use "numpy.savetxt" instead.')







from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset      = spm1d.data.uv1d.t2.PlantarArchAngle()
dataset      = spm1d.data.uv1d.t2.SimulatedTwoLocalMax()
YA,YB        = dataset.get_data()  #normal and fast walking



#(1) Conduct t test:
alpha      = 0.05
t          = spm1d.stats.ttest2(YB, YA, equal_var=True)
ti         = t.inference(alpha, two_tailed=False, interp=True)
print ti



#(2) Plot:
pyplot.close('all')
### plot mean and SD:
pyplot.figure( figsize=(8, 3.5) )
ax     = pyplot.axes( (0.1, 0.15, 0.35, 0.8) )
spm1d.plot.plot_mean_sd(YA)
spm1d.plot.plot_mean_sd(YB, linecolor='r', facecolor='r')
ax.axhline(y=0, color='k', linestyle=':')
ax.set_xlabel('Time (%)')
ax.set_ylabel('Plantar arch angle  (deg)')
### plot SPM results:
ax     = pyplot.axes((0.55,0.15,0.35,0.8))
ti.plot()
ti.plot_threshold_label(fontsize=8)
ti.plot_p_values(size=10, offsets=[(0,0.3)])
ax.set_xlabel('Time (%)')
pyplot.show()




from setuptools import setup



long_description = '''
**spm1d** is a Python package for <b>one-dimensional Statistical Parametric Mapping<b>.
spm1d uses <b>Random Field Theory</b> expectations regarding the behavior of smooth,
one-dimensional Gaussian fields to make statistical inferences regarding a set of
one-dimensional measurements.
'''

setup(
	name             = 'spm1d',
	version          = '0.3.0.1',
	description      = 'One-Dimensional Statistical Parametric Mapping',
	author           = 'Todd Pataky',
	author_email     = 'spm1d.mail@gmail.com',
	url              = 'https://github.com/0todd0000/spm1d',
	download_url     = 'https://github.com/0todd0000/spm1d/archive/master.zip',
	packages         = ['spm1d'],
	package_data     = {'spm1d' : ['examples/*.*', 'data/*.*'] },
	include_package_data = True,
	long_description = long_description,
	keywords         = ['statistics', 'time series analysis'],
	classifiers      = [],
	install_requires = ["numpy", "scipy", "matplotlib", "rft1d"]
) 



import os
from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset      = spm1d.data.mv1d.cca.Dorn2012()
Y,x          = dataset.get_data()  #A:slow, B:fast
print dataset



#(1) Conduct test:
alpha        = 0.05
X2           = spm1d.stats.cca(Y, x)
X2i          = X2.inference(0.05)



#(2) Plot:
pyplot.close('all')
X2i.plot()
X2i.plot_p_values()
pyplot.show()






import numpy as np
from matplotlib import pyplot
import spm1d





#(0) Example one-way (0D):
dataset  = spm1d.data.uv0d.anova1rm.Abdi2010()
dataset  = spm1d.data.uv0d.anova1rm.Groceries()
dataset  = spm1d.data.uv0d.anova1rm.Imacelebrity()
dataset  = spm1d.data.uv0d.anova1rm.Southampton1rm()
y,A,SUBJ = dataset.get_data()
print dataset



#(1) Conduct ANOVA:
F        = spm1d.stats.anova1rm(y, A, SUBJ, equal_var=False)
Fi       = F.inference(0.05)
print Fi



#(2) Plot design:
pyplot.close('all')
design   = spm1d.stats.anova.designs.ANOVA1rm(A, SUBJ)
design.plot()
pyplot.show()





import numpy as np
import designs,solvers
from .. import _datachecks, _reml


### ONE-WAY DESIGNS ##############

def anova1(Y, A=None, equal_var=False):
	'''
	One-way ANOVA.
	
	:Parameters (Option 1):
		- *Y* --- A list or tuple of (J x Q) numpy arrays
		- *equal_var* --- If *True*, equal group variance will be assumed


	:Parameters (Option 2):
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer group labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	
	:Returns:
		- F : An **spm1d._spm.SPM_F** instance
	
	:Example:
	
	>>> F = spm1d.stats.anova1((Y0,Y1,Y2))
	>>> Fi = F.inference(alpha=0.05)
	>>> Fi.plot()
	'''
	if isinstance(Y, (list,tuple)):
		_datachecks.check('anova1', Y)
		A   = np.array([[i]*y.shape[0] for i,y in enumerate(Y)]).flatten()
		Y   = np.hstack(Y) if Y[0].ndim==1 else np.vstack(Y)
	design  = designs.ANOVA1(A)
	### simplified design matrix:
	# design.set_simplified_design(with_const=False, difference_contrasts=True)
	### this is here just for reference and future use:
	### it is possible to compute the F statistic using a simpler alternative
	### design matrix, but this matrix complicates non-sphericity corrections
	solver  = solvers.ANOVA1(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()[0]
	if not equal_var:
		Y,X,r = solver.Y, solver.X, solver.eij
		Q,C   = design.A.get_Q(), design.contrasts[0].C.T
		f.df  = _reml.estimate_df_anova1(Y, X, r, Q, C)
	return f



def anova1rm(Y, A, SUBJ, equal_var=False):
	'''
	One-way repeated-measures ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer group labels
		- *SUBJ* --- (J x 1) vector of subject labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	
	:Returns:
		- F : An **spm1d._spm.SPM_F** instance
	
	:Example:
	
	>>> Y = np.random.randn(9, 101)
	>>> A = np.array([1,1,1, 2,2,2, 3,3,3])
	>>> SUBJ = np.array([1,2,3, 1,2,3, 1,2,3])
	>>> F = spm1d.stats.anova1(Y, A, SUBJ)
	>>> Fi = F.inference(alpha=0.05)
	>>> Fi.plot()
	'''
	design  = designs.ANOVA1rm(A, SUBJ)
	solver  = solvers.ANOVA1rm(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()[0]
	if not equal_var:
		Y,X,r = solver.Y, solver.X, solver.eij
		Q,C   = design.A.get_Q(), design.contrasts[0].C.T
		f.df  =  _reml.estimate_df_anova1(Y, X, r, Q, C)
		# QS    = design.S.get_Q()
		# QQ    = Q+ QS
		# f.df  = _reml.estimate_df_anova1(Y, X, r, QQ, C)
	return f






### TWO-WAY DESIGNS ##############


def anova2(Y, A, B, equal_var=False):
	'''
	Two-way ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of three **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Interaction AB
	'''
	design  = designs.ANOVA2(A, B)
	solver  = solvers.ANOVA2(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	if not equal_var:
		Y,X,r   = solver.Y, solver.X, solver.eij
		QA,QB,C = design.A.get_Q(), design.B.get_Q(), [c.C.T for c in design.contrasts]
		Q       = QA + QB
		u1,u2   = _reml.estimate_df_anova2(Y, X, r, Q, C)
		for ff,u in zip(f,u1):
			ff.df = u,u2
	return f


def anova2nested(Y, A, B, equal_var=False):
	'''
	Two-way nested ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B (nested in A)
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of two **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B

	:Note:
		- there is no interaction term in nested designs.
	'''
	design  = designs.ANOVA2nested(A, B)
	solver  = solvers.ANOVA2nested(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	if not equal_var:
		Y,X,r   = solver.Y, solver.X, solver.eij
		QA,QB,C = design.A.get_Q(), design.B.get_Q(), [c.C.T for c in design.contrasts]
		Q       = QA + QB
		u1,u2   = _reml.estimate_df_anova2(Y, X, r, Q, C)
		f[0].df = u1[0], u1[1]
		f[1].df = u1[1], u2
	return f


def anova2rm(Y, A, B, SUBJ, equal_var=True):
	'''
	Two-way repeated-measures ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B
		- *SUBJ* --- (J x 1) vector of integer subject labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of three **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Interaction AB
			
	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	design  = designs.ANOVA2rm(A, B, SUBJ)
	solver  = solvers.ANOVA2rm(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	# if not equal_var:
	# 	Y,X,r   = solver.Y, solver.X, solver.eij
	# 	QA,QB,C = design.A.get_Q(), design.B.get_Q(), [c.C.T for c in design.contrasts]
	# 	Q       = QA + QB
	# 	u1,u2   = _reml.estimate_df_anova2(Y, X, r, Q, C)
	# 	for ff,u in zip(f,u1):
	# 		ff.df = u,u2
	return f


def anova2onerm(Y, A, B, SUBJ, equal_var=True):
	'''
	Two-way ANOVA with repeated-measures on one factor.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B (the repeated-measures factor)
		- *SUBJ* --- (J x 1) vector of integer subject labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of three **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Interaction AB
			
	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	design  = designs.ANOVA2onerm(A, B, SUBJ)
	solver  = solvers.ANOVA2onerm(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	return f


### THREE-WAY DESIGNS ##############


def anova3(Y, A, B, C, equal_var=False):
	'''
	Three-way ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B
		- *C* --- (J x 1) vector of integer labels for Factor C
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of seven **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Main effect C
			4. Interaction AB
			5. Interaction AC
			6. Interaction BC
			7. Interaction ABC
	'''
	design  = designs.ANOVA3(A, B, C)
	solver  = solvers.ANOVA3(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	if not equal_var:
		Y,X,r   = solver.Y, solver.X, solver.eij
		QA,QB,QC,C = design.A.get_Q(), design.B.get_Q(), design.C.get_Q(), [c.C.T for c in design.contrasts]
		Q       = QA + QB + QC
		u1,u2   = _reml.estimate_df_anova2(Y, X, r, Q, C)
		for ff,u in zip(f,u1):
			ff.df = u,u2
	return f

def anova3nested(Y, A, B, C, equal_var=False):
	'''
	Three-way fully nested ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B (nested in A)
		- *C* --- (J x 1) vector of integer labels for Factor C (nested in B)
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of three **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Main effect C

	:Note:
		- there are no interaction terms in fully-nested designs.
	'''

	design  = designs.ANOVA3nested(A, B, C)
	solver  = solvers.ANOVA3nested(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	if not equal_var:
		Y,X,r   = solver.Y, solver.X, solver.eij
		QA,QB,QC,C = design.A.get_Q(), design.B.get_Q(), design.C.get_Q(), [c.C.T for c in design.contrasts]
		Q       = QA + QB + QC
		u1,uE   = _reml.estimate_df_anova2(Y, X, r, Q, C)
		uA,uB,uC = u1
		f[0].df = uA, uB
		f[1].df = uB, uC
		f[2].df = uC, uE
	return f

	
def anova3onerm(Y, A, B, C, SUBJ, equal_var=True):
	'''
	Three-way ANOVA with repeated-measures on one factor.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B
		- *C* --- (J x 1) vector of integer labels for Factor C (the repeated-measures factor)
		- *SUBJ* --- (J x 1) vector of integer subject labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of seven **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Main effect C
			4. Interaction AB
			5. Interaction AC
			6. Interaction BC
			7. Interaction ABC
			
	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	design  = designs.ANOVA3onerm(A, B, C, SUBJ)
	solver  = solvers.ANOVA3onerm(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	return f

def anova3tworm(Y, A, B, C, SUBJ, equal_var=True):
	'''
	Three-way ANOVA with repeated-measures on two factors.
	
	:Parameters:
		- *Y* --- (J x Q) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *B* --- (J x 1) vector of integer labels for Factor B (a repeated-measures factor)
		- *C* --- (J x 1) vector of integer labels for Factor C (a repeated-measures factor)
		- *SUBJ* --- (J x 1) vector of integer subject labels
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- List of seven **spm1d._spm.SPM_F** instances in the following order:
			1. Main effect A
			2. Main effect B
			3. Main effect C
			4. Interaction AB
			5. Interaction AC
			6. Interaction BC
			7. Interaction ABC
			
	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	design  = designs.ANOVA3tworm(A, B, C, SUBJ)
	solver  = solvers.ANOVA3tworm(Y, design.X, design.contrasts)
	f       = solver.compute_test_statistics()
	return f




import numpy as np
from matplotlib import pyplot
from contrasts import Contrast,ContrastUnbalanced,CompoundContrast


class ModelBuilder(object):
	def __init__(self, labels=[], labels_subj=[]):
		self.labels  = list(labels)
		self.slabels = list(labels_subj)
		self.n       = len(self.labels)   #number of main factors
		self.nS      = len(self.slabels)  #number of subject factors
		self.ncol    = 0  #number of main columns
		self.ncolS   = 0  #number of subject columns
		self.ncolT   = 0  #total number of columns
		self.colD    = dict(zip(self.labels,  [None]*self.n))
		self.colSD   = dict(zip(self.slabels, [None]*self.nS))
		self.XD      = dict(zip(self.labels,  [None]*self.n))
		self.XSD     = dict(zip(self.slabels, [None]*self.n))


	def add_main_columns(self, label, X):
		self.XD[label]   = X
		i0,n             = self.ncol, X.shape[1]
		self.colD[label] = np.arange(i0, i0+n)
		self.ncol       += n
		self.ncolT      += n



	def add_subj_columns(self, label, X):
		self.XSD[label]   = X
		i0,n              = self.ncolS, X.shape[1]
		self.colSD[label] = np.arange(i0, i0+n)
		self.ncolS       += n
		self.ncolT       += n


	def get_contrast(self, label):
		cols      = self.colD[label]
		n         = len(cols)
		C         = np.zeros( (n,self.ncolT) )
		C[:,cols] = np.eye(n)
		return Contrast(C)

	def get_contrast_compound(self, label, slabel):
		if isinstance(slabel,list):
			return self.get_contrast_compound3(label, slabel)
		cols,scols  = self.colD[label], self.colSD[slabel] + self.ncol
		n,ns        = len(cols), len(scols)
		C,CS        = np.zeros( (n,self.ncolT) ), np.zeros( (ns,self.ncolT) )
		C[:,cols]   = np.eye(n)
		CS[:,scols] = np.eye(ns)
		return CompoundContrast(np.vstack([C,CS]), n)

	def get_contrast_compound3(self, label, slabels):
		slabel0,slabel1    = slabels
		cols,scols0,scols1 = self.colD[label], self.colSD[slabel0] + self.ncol, self.colSD[slabel1] + self.ncol
		n,ns0,ns1          = len(cols), len(scols0), len(scols1)
		C,CS0,CS1          = np.zeros( (n,self.ncolT) ), np.zeros( (ns0,self.ncolT) ), np.zeros( (ns1,self.ncolT) )
		C[:,cols]          = np.eye(n)
		CS0[:,scols0]      = np.eye(ns0)
		CS1[:,scols1]      = np.eye(ns1)
		CS                 = CS0 + CS1
		return CompoundContrast(np.vstack([C,CS]), n)

	def get_contrast_difference(self, label):
		cols      = self.colD[label]
		n         = len(cols)
		C         = np.zeros( (n-1,self.ncolT) )
		for i in range(n-1):
			i0,i1 = cols[i], cols[i+1]
			C[i,[i0,i1]] = [1,-1]
		return Contrast(C)

	def get_contrast_unbalanced(self, label):
		return ContrastUnbalanced( self.colD[label].tolist()  )
	
	def get_design_matrix(self):
		X   = np.hstack([self.XD[label]  for label in self.labels])
		if self.nS>0:
			XS  = np.hstack([self.XSD[label] for label in self.slabels])
			X   = np.hstack([X,XS])
		return X
		
		


	



import os
import numpy as np
from matplotlib import pyplot
import spm1d

# load data (30 curves, arbitrary length):
dir0         = os.path.dirname(__file__)
fname        = os.path.join(dir0, 'data', 'ex_kinematics_for_interpolation.npy')
Y0           = np.load(fname)   #30 curves


# interpolate to 101 points:
Y            = spm1d.util.interp(Y0, 101)


# plot:
pyplot.close('all')
pyplot.figure(figsize=(8,3.5))
ax0          = pyplot.axes((0.1,0.15,0.35,0.8))
ax1          = pyplot.axes((0.55,0.15,0.35,0.8))
[ax0.plot(y, 'k')  for y in Y0]
pyplot.plot(Y.T, 'k')
ax0.text(0.5, 0.9, 'Before interpolation', size=14, transform=ax0.transAxes, ha='center')
ax1.text(0.5, 0.9, 'After interpolation', size=14, transform=ax1.transAxes, ha='center')
pyplot.show()

